
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 0.5.2 Generate Ramp */

#include "PhotoshopShell.h"


///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors
	long	height, width;									//pixel dimensions of image
	long	x, y;											//loop counters to traverse image
	RGBPixel *line = NULL;									//memory buffer
	float	fraction;
	Boolean	Horiz = FALSE;									//true for horizontal ramp, false for vertical

	GetOriginalDimensions(&width, &height);					//find out how big the image is

	line = CreateAPointer(width, sizeof(RGBPixel));			//Allocate memory for one row

	//ignore current image contents, create linear ramp
	for (y = 0; y < height; y++)
	{
		if (TestAbortSetProgressBar((float)y / (float)height))	//See if user canceled
		{
			result = userCanceledErr;
			goto mainexit;
		}

		ReadOriginalLine(y, line);							//read a line
		for (x = 0; x < width; x++)
		{	
			if (Horiz)
				fraction = (float)x/(float)width;
			else
				fraction = (float)y/(float)height;
			line[x].red =   (float)255.0*fraction;
			line[x].green = (float)255.0*fraction;
			line[x].blue =  (float)255.0*fraction;
		}//for x
		
		WriteResultLine(y, line);							//write changed line back
	}//for y

mainexit:													//clean up everything we allocated
	if (line)
		DisposeAPointer(line);

	return result;											//pass any errors back
}//MainUserEntry

