
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 1.1.5.1 Save Histogram */

#include "PhotoshopShell.h"


///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happen
	long	height, width;									//pixel dimensions of image
	long	x, y;											//loop counters to traverse image
	long	bright;											//index for histogram
	long	Histogram[256];									//fixed number of bins
	RGBPixel *line = NULL;									//memory buffer

	GetOriginalDimensions(&width, &height);					//find out how big the image is

	line = CreateAPointer(width, sizeof(RGBPixel));			//Allocate memory for one row

	for (bright = 0; bright < 256; bright++)
		Histogram[bright] = 0;								//clear the array			

	for (y = 0; y < height; y++) // traverse image line at a time
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{	//See if user canceled
			result = userCanceledErr;
			goto mainexit;
		}
		ReadOriginalLine(y, line);							//read a line
		for (x = 0; x < width; x++) // traverse each line a pixel at a time
		{
			bright = (long)((line[x].red + line[x].green + line[x].blue)/3); // average brightness
			Histogram[bright]++;							//Increment histogram bin count
		}//for x
	}//for y
	
	{	// write the data to a text file
		long fref = 0; // file ID number
		if (CreateTextFileToWrite("C:\\histogram.txt", &fref) == 0);//location to write file
		{
			for (bright = 0; bright < 256; bright++)
				WriteANumber(fref, (float)Histogram[bright]);
				// could use (float)Histogram[bright]/(float)(height*width) for area fraction
			CloseTextFile(fref);
		}// if file was successfully opened
	}

mainexit:													//clean up everything we allocated
	if (line)
		DisposeAPointer(line);

	return result;											//pass any errors back
}//MainUserEntry

