
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 1.1.5.4 Contrast Stretch*/

#include "PhotoshopShell.h"

#define bins 1000 //number of bins for histogram controls clipping precision

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;							//accumulate any errors that happened
	long	height, width;							//pixel dimensions of image
	long	x, y;									//loop counters to traverse image
	float	bright;									//intensity or luminance
	long	index, bottom=0, top=bins-1;			//index into histo array
	float	sum, area;
	float	clip = (float)0.0025;					//clipping fraction = 1/4% at each end
	float	gamma = 1.5;							//gamma adjustment
	float	lower, upper, scale;					//limits for rescaling
	RGBPixel *line = NULL;							//memory buffer
	long	*Histo;									//pointer to histogram array
	
	GetOriginalDimensions(&width, &height);			//find out how big the image is
	area = (float)(width * height);					//total pixel count
	line = CreateAPointer(width, sizeof(RGBPixel));	//allocate memory for one row
	Histo = CreateAPointer(bins, sizeof(long));		//allocate memory for histogram

	for (index = 0; index < bins; index++)			//initialize the array
		Histo[index] = 0;
		
	//first build histogram - traverse the image line by line
	for (y = 0; y < height; y++)
	{
		if (TestAbortSetProgressBar((float)y/(float)height*2))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		
		ReadOriginalLine(y, line);				//read a line
		
		for (x = 0; x < width; x++) // traverse the line pixel by pixel
		{	//calculate intensity as average of red, green and blue
			bright = (line[x].red + line[x].green +line[x].blue)/3;
			index = (long)(bins * bright/255.0); //calculate index to array
			if (index > bins) index=bins;// handle case of bright=255
			Histo[index]++;//increment array
		}//for x
	}//for y

	// scan histogram for clipping points
	sum = 0.0;
	for (index = 0; index < bins; index++)
	{
		sum += (float)Histo[index]; // cumulative pixel count
		if (sum/area <= clip) bottom = index;
		if (sum/area < (1.0 - clip)) top = index;
	}

	lower = (float)255.0*(float)bottom/bins; //calculate limits on 0..255 scale
	upper = (float)255.0*(float)top/(float)bins;
	scale = (float)255.0/(upper - lower);

	//second pass - rescale L values
	for (y = 0; y < height; y++)//line by line
	{
		if (TestAbortSetProgressBar((float)(y+height)/(float)height*2))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		
		ReadOriginalLine(y, line);				//read a line
		
		for (x = 0; x < width; x++)//pixel by pixel
		{
			float A,B,valu;	// changing to LAB to modify L leaving colors alone
			
			bright = (line[x].red + line[x].green +line[x].blue)/3;// L value
			A = (line[x].red - line[x].green);
			B = (line[x].red + line[x].green)/2 - line[x].blue;
			
			bright = scale * (bright - lower);//rescaled L value
			if (bright < 0) bright = 0;// clip under- and overflow
			if (bright > 255) bright = 255;
			
			// optional gamma adjustment
			valu = (bright+1)/(float)256.0; // in range 0..1 but not exactly zero
			valu = (float)exp(log(valu)/gamma);//apply gamma function
			
			bright = (valu * (float)256.0) - 1; // 0..255
			
			line[x].red =   bright + A/2 + B/3;//recover the RGB values
			line[x].green = bright - A/2 + B/3;
			line[x].blue =  bright - 2 * B/3;
			
		}//for x
		WriteResultLine(y, line);				//write changed line back
	}//for y

mainexit:										//clean up everything we allocated
	if (line)
		DisposeAPointer(line);
	if (Histo)
		DisposeAPointer(Histo);

	return result;								//pass any errors back
}//MainUserEntry

