
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 1.1.5.6 Contrast Equalization */

#include "PhotoshopShell.h"

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;							//accumulate any errors that happened
	long	height, width;							//pixel dimensions of image
	long	x, y;									//loop counters to traverse image
	float	bright;									//mean intensity
	long	index;									//index to histogram array
	float	sum, area;
	float	value, fraction;						//used in calculation of new intensity
	RGBPixel *line = NULL;							//memory buffer
	long	Histo[256];								//histogram array
	float	xfer[256];								//lookup table or transfer function
	
	GetOriginalDimensions(&width, &height);			//find out how big the image is
	area = (float)(width * height);					//total pixel count
	line = CreateAPointer(width, sizeof(RGBPixel));	//allocate memory for one row

	for (index = 0; index < 256; index++)			//initialize histogram
		Histo[index] = 0;
		
	//first pass - build histogram
	for (y = 0; y < height; y++)//line by line
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		
		ReadOriginalLine(y, line);				//read a line
		
		for (x = 0; x < width; x++)//pixel by pixel
		{
			bright = (line[x].red + line[x].green +line[x].blue)/3;// average intensity
			index = (long)(bright); // index to histogram
			if (index > 255) index=255;// handle case of bright=255
			Histo[index]++;	//increment array
		}//for x
	}//for y

	// convert histogram to transfer function
	sum = 0.0;
	for (index = 0; index < 256; index++)
	{
		sum += (float)Histo[index]; // cumulative pixel count
		xfer[index] = 255 * (sum/area);
	}

	/*alternate transfer function
	//emphasizes center of histogram rather than uniform linear equalization
	for (index = 0; index < 256; index++)
	{	if (index <=127)
		{	value = (float)index/127.5;
			value = sqrt(value)/2;
		}
		else
		{	value = (float)(255-index)/127.5;
			value = 1.0 - sqrt(value)/2;
		}
		xfer[index] = 255 * value;
	}*/

	//second pass - rescale L values
	for (y = 0; y < height; y++)//line by line
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadOriginalLine(y, line);				//read a line
		
		for (x = 0; x < width; x++)//pixel by pixel
		{
			float A,B;//preserve color values
			bright = (line[x].red + line[x].green +line[x].blue)/3;
			A = (line[x].red - line[x].green);
			B = (line[x].red + line[x].green)/2 - line[x].blue;
			index = (long)bright;//index to transfer function
			fraction = bright - (float)index;//for interpolation
			value = (1 - fraction) * xfer[index];//result from lookup table
			if (index < 255) value += fraction * xfer[index+1];//with interpolation
			bright = value;//new intensity
			line[x].red =   bright + A/2 + B/3;//convert back to RGB
			line[x].green = bright - A/2 + B/3;
			line[x].blue =  bright - 2 * B/3;
			
		}//for x
		WriteResultLine(y, line);				//write changed line back
	}//for y

mainexit:										//clean up everything we allocated
	if (line)
		DisposeAPointer(line);

	return result;								//pass any errors back
}//MainUserEntry

