
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 1.2.4.1 Neutral Color Correction */

#include "PhotoshopShell.h"

#define bins 1000 //number of bins for histogram controls clipping precision

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;							//accumulate any errors that happened
	long	height, width;							//pixel dimensions of image
	long	x, y;									//loop counters to traverse image
	float	bright;									//mean intensity
	long	index, bottom=0, top=bins-1;			//histogram index and limits
	float	sum, area;
	float	clip = (float)0.01;						//1% at each end
	float	DarkR, DarkG, DarkB, BriteR, BriteG, BriteB;//limits for each channel
	long	Darkcount, Britecount;
	float	lower, upper, scaleR, scaleG, scaleB;	//scaling factors for conversion
	RGBPixel *line = NULL;							//memory buffer for one line of pixels
	long	*Histo;									//pointer for histogram array
	
	GetOriginalDimensions(&width, &height);			//find out how big the image is
	area = (float)(width * height);					//total pixel count
	line = CreateAPointer(width, sizeof(RGBPixel));	//allocate memory for one row
	Histo = CreateAPointer(bins, sizeof(long));		//allocate memory for histogram

	for (index = 0; index < bins; index++)			//initialize array
		Histo[index] = 0;
		
	//first pass - build histogram
	for (y = 0; y < height; y++)//line by line
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		
		ReadOriginalLine(y, line);				//read a line
		
		for (x = 0; x < width; x++)//pixel by pixel
		{	// average RGB, could also use other weighting
			bright = (line[x].red + line[x].green +line[x].blue)/3;
			index = (long)(bins * bright/255.0);//index to array
			if (index > bins) index=bins;// handle case of bright=255
			Histo[index]++;//increment histogram
		}//for x
	}//for y

	// scan histogram for clipping points
	sum = 0.0;
	for (index = 0; index < bins; index++)
	{	sum += (float)Histo[index]; // cumulative pixel count
		if (sum/area <= clip) bottom = index;
		if (sum/area < (1.0 - clip)) top = index;
	}
	lower = (float)255.0*(float)bottom/bins; //calculate limits on 0..255 scale
	upper = (float)255.0*(float)top/bins;

	//second pass - read average RGB values for bright and dark pixels
	DarkR = DarkG = DarkB = BriteR = BriteG = BriteB = 0.0;// initialize
	Darkcount = Britecount = 0;
	for (y = 0; y < height; y++)//line by line
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		
		ReadOriginalLine(y, line);
		
		for (x = 0; x < width; x++)//pixel by pixel
		{
			bright = (line[x].red + line[x].green +line[x].blue)/3;
			if (bright <= lower) // below lower clipping limit
			{	DarkR += line[x].red;// construct sums for averages
				DarkG += line[x].green;
				DarkB += line[x].blue;
				Darkcount ++;
			}
			if (bright >= upper) // above upper clipping limit
			{	BriteR += line[x].red; // construct sums for averages
				BriteG += line[x].green;
				BriteB += line[x].blue;
				Britecount ++;
			}
		}//for x
	}//for y
	DarkR /= Darkcount; DarkG /= Darkcount; DarkB /= Darkcount; // averages
	BriteR/=Britecount; BriteG/=Britecount; BriteB/=Britecount;
	scaleR = (float)255.0/(BriteR - DarkR); // scale factors for rescaling
	scaleG = (float)255.0/(BriteG - DarkG);
	scaleB = (float)255.0/(BriteB - DarkB);

	//third pass - rescale RGB values
	for (y = 0; y < height; y++)// one more time!
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		
		ReadOriginalLine(y, line);				//read a line
		
		for (x = 0; x < width; x++)
		{	float Rvalu, Gvalu, Bvalu;
			Rvalu = line[x].red; // original RGB values
			Gvalu = line[x].green;
			Bvalu = line[x].blue;
			Rvalu = scaleR * (Rvalu - DarkR); // rescaled between limits
			Gvalu = scaleG * (Gvalu - DarkG);
			Bvalu = scaleB * (Bvalu - DarkB);
			if (Rvalu > 255) Rvalu = 255; if (Rvalu < 0) Rvalu = 0; // clip over/underflow
			if (Gvalu > 255) Gvalu = 255; if (Gvalu < 0) Gvalu = 0;
			if (Bvalu > 255) Bvalu = 255; if (Bvalu < 0) Bvalu = 0;
			line[x].red = Rvalu; // save new values
			line[x].green = Gvalu;
			line[x].blue = Bvalu;
		}//for x
		
		WriteResultLine(y, line);				//write changed line back
	}//for y

mainexit:										//clean up everything we allocated
	if (line)
		DisposeAPointer(line);
	if (Histo)
		DisposeAPointer(Histo);

	return result;								//pass any errors back
}//MainUserEntry

