
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 1.2.4.2 Convert to Gray Scale */

#include "PhotoshopShell.h"

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;							//accumulate any errors that happened
	long	height, width;							//pixel dimensions of image
	long	x, y;									//loop counters to traverse image
	float	bright;									//intensity or luminance value
	float	RWeight, GWeight, BWeight;				//weights for r, g, b
	RGBPixel *line = NULL;							//memory buffer for one line of pixels
	
	GetOriginalDimensions(&width, &height);			//find out how big the image is

	line = CreateAPointer(width, sizeof(RGBPixel));	//allocate memory for one line

	RWeight = (float)0.2125; GWeight = (float)0.7154; BWeight = (float)0.0721;
	// These weights produce the usual luminance and are useful for testing
	// Optionally, could read the weight values from a text file
	// Also it may be useful to normalize them to sum to 1.0
	{	long fref; //file reference number
		float total;
		if (OpenTextFileToRead("C:\\RGB.txt", &fref) != noErr)
		{
			result = fnfErr; // return file not found error
			goto mainexit;
		}
		else
		{	
			result = ReadANumber(fref, &RWeight);
			result = ReadANumber(fref, &GWeight);
			result = ReadANumber(fref, &BWeight);
			CloseTextFile(fref);
			if (result != noErr)
				goto mainexit;
			else
			{	total = RWeight + GWeight + BWeight;
				RWeight = RWeight / total;
				GWeight = GWeight / total;
				BWeight = BWeight / total;
			}
		}
	}
	
	for (y = 0; y < height; y++)//traverse image line by line
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		
		ReadOriginalLine(y, line);				//read a line
		
		for (x = 0; x < width; x++)//traverse line pixel by pixel
		{	//assign arbitrary weight factors to red, green and blue
			bright = RWeight*line[x].red + GWeight*line[x].green + BWeight*line[x].blue;
			if (bright<0) bright = 0;
			if (bright>255) bright = 255; //depending on weights, may need to clip
			line[x].red = line[x].green = line[x].blue = bright; //same for all channels
			//write back same value for R, G, B to produce gray result
		}
		
		WriteResultLine(y, line);				//write changed line back
	}//for y

mainexit:										//clean up everything we allocated
	if (line)
		DisposeAPointer(line);

	return result;								//pass any errors back
}//MainUserEntry

