
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 1.3.3.1 Correct Vignetting */

#include "PhotoshopShell.h"

double	XpwrY(double base, double power);// calculate X^^Y
double	XpwrY(double base, double power)
{	double	temp = log(base);
	temp *= power;
	temp = exp(temp);
	return(temp);
}// XpwrY

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;							//accumulate any errors that happened
	long	height, width;							//pixel dimensions of image
	long	x, y;									//loop counters to traverse image
	float	dx, dy, radius, maxrad, xcenter, ycenter;//location of center and distance from it
	float	bright, value;							//brightness value
	float	boost = 48;								// difference between center and corner
	float	power = 2.5;							// power law for curvature
	RGBPixel *line = NULL;							//memory buffer

	GetOriginalDimensions(&width, &height);			//find out how big the image is
	xcenter = (float)width/2;
	ycenter = (float)height/2;						//center coordinates
	maxrad = (float)sqrt(xcenter*xcenter+ycenter*ycenter);	//radius to corner
	
	line = CreateAPointer(width, sizeof(RGBPixel));	//allocate memory for one row

	for (y = 0; y < height; y++)//traverse image line by line
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		
		dy = (float)y - ycenter;				//vertical distance from center
		ReadOriginalLine(y, line);				//read a line
		
		for (x = 0; x < width; x++)//traverse line pixel by pixel
		{
			float r,g,b;						//red, green and blye values
			dx = (float)x - xcenter;			//horizontal distance from center
			radius = (float)sqrt(dx*dx+dy*dy) / maxrad; // 0..1 (normalized radius from center)
			value = boost * (float)XpwrY(radius, power);// amount to increase brightness
			
			r = line[x].red; 					//get pixel r, g, b values
			g = line[x].green;
			b = line[x].blue;
			bright = (float)0.25*r + (float)0.65*g + (float)0.10*b;// or other weights to calculate brightness
			if (bright > 0)
			{
				value = (value+bright) / bright;//amount to boost each value
				r *= value; if (r>255) r=255;	//clip to prevent overflow
				b *= value; if (b>255) b=255;
				g *= value; if (g>255) g=255;
				line[x].red = r;				 //boost with ratios, preserve colors
				line[x].green = g;
				line[x].blue = b;
			}
		}
		WriteResultLine(y, line);				//write changed line back
	}//for y

mainexit:										//clean up everything we allocated
	if (line)
		DisposeAPointer(line);

	return result;								//pass any errors back
}//MainUserEntry

