
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 2.1.10.1 Zoom */

#include "PhotoshopShell.h"

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	x, y;											//loop counters to traverse image
	long	xcenter, ycenter;								//coordinates of center point
	float	scale;											//amount to zoom
	float	OrigX, OrigY;									//coordinates of originating point
	long	FetchX, FetchY;									//integer address of pixel to read
	float	dx, dy;											//fractional part of address to interpolate
	RGBPixel *LineA= NULL, *LineB= NULL, *Result= NULL;		//line pointers to read and write
	long	fref;											//file pointer to read zoom factor

	GetOriginalDimensions(&width, &height);					//find out how big the image is
	xcenter = width / 2;									//center coordinates
	ycenter = height / 2;
	
	Result = CreateAPointer(width, sizeof(RGBPixel));		//to write one line at a time
	LineA = CreateAPointer(width, sizeof(RGBPixel));		//and read two lines for interpolation
	LineB = CreateAPointer(width, sizeof(RGBPixel));

	scale = (float)1.667;									// value defined here for testing but read from file below

	if (OpenTextFileToRead("C:\\zoom.txt", &fref) != noErr)
	{	result = fnfErr;
		goto mainexit;
	}
	else
	{	result = ReadANumber(fref, &scale);;
		CloseTextFile(fref);
		if (result != noErr)
			goto mainexit;
	}

	for (y = 0; y < height; y++) 						// traverse destination image line by line
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{	//See if user canceled and update progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		OrigY = (float)ycenter + (y - ycenter) / scale;	//y address of originating point
		FetchY = (long)OrigY;							//integer part of y address
		dy = OrigY - (float)FetchY;						//fractional part of y address
		if ((FetchY >=0) && (FetchY < height-1))		//if location is within image area
		{
			ReadOriginalLine(FetchY, LineA);			//read two lines bracketing address
			ReadOriginalLine(FetchY+1, LineB);
			for (x = 0; x < width; x++)					//traverse line pixel by pixel
			{
				OrigX = (float)xcenter + (x - xcenter) / scale;	//x address of originating point
				FetchX = (long)OrigX;					//integer part of x address
				dx = OrigX - (float)FetchX;				//fractional part of x address
				if ((FetchX >= 0) && (FetchX < width-1))// if location is within image area
				{
					float valA, valB;					//for horizontal interpolation
					valA = (1-dx) * LineA[FetchX].red + dx * LineA[FetchX+1].red; //interpolate on each line
					valB = (1-dx) * LineB[FetchX].red + dx * LineB[FetchX+1].red; //and for each color channel
					Result[x].red = (1-dy) * valA + dy * valB;// perform vertical interpolation
					valA = (1-dx) * LineA[FetchX].green + dx * LineA[FetchX+1].green;
					valB = (1-dx) * LineB[FetchX].green + dx * LineB[FetchX+1].green;
					Result[x].green = (1-dy) * valA + dy * valB;
					valA = (1-dx) * LineA[FetchX].blue + dx * LineA[FetchX+1].blue;
					valB = (1-dx) * LineB[FetchX].blue + dx * LineB[FetchX+1].blue;
					Result[x].blue = (1-dy) * valA + dy * valB;
				}
			}
		}
		else
		{	
			for (x = 0; x < width; x++)
				Result[x].red = Result[x].green = Result[x].blue = 0; //point is not in original image
		}

		WriteResultLine(y, Result);
	}//for y
	
mainexit:	//clean up everything we allocated
	if (Result) DisposeAPointer(Result);
	if (LineA) DisposeAPointer(LineA);
	if (LineB) DisposeAPointer(LineB);

	return result;											//pass any errors back
}//MainUserEntry

