
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 2.1.10.1 Rotate */

#include "PhotoshopShell.h"

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	x, y;											//loop counters to traverse image
	long	xcenter, ycenter, xshift, yshift;				//location of center point and x,y offsets
	float	angle;											//rotation angle
	float	xorig, yorig, xprime, yprime;					//address of originating point
	long	x2, y2;											//integer part of address
	float	dx, dy, valA, valB;								//fractional part of address and partial interpolation
	RGBPixel *LineA= NULL, *LineB= NULL, *Result= NULL;		//line pointers to read and write
	long	fref;											//file ID to read angle

	GetOriginalDimensions(&width, &height);					//find out how big the image is
	xcenter = width / 2;
	ycenter = height / 2;
	
	Result = CreateAPointer(width, sizeof(RGBPixel));
	LineA = CreateAPointer(width, sizeof(RGBPixel));
	LineB = CreateAPointer(width, sizeof(RGBPixel));

	//angle = 37.5;	// value defined here for testing but read from file below

	if (OpenTextFileToRead("C:\\rotate.txt", &fref) != noErr)
	{	result = fnfErr;
		goto mainexit;
	}
	else
	{	result = ReadANumber(fref, &angle);;
		CloseTextFile(fref);
		if (result != noErr)
			goto mainexit;
	}

	angle *= (float)(3.14159265 / 180);							// convert to radians
	
	for (y = 0; y < height; y++)							// traverse destination image line by line
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{	//See if user canceled and update progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		for (x = 0; x < width; x++)//traverse each line of destination image pixel by pixel
		{
			xshift = x-width/2; yshift = y-height/2;//distance from center of image
			xprime = (float)xshift * (float)cos(angle) + (float)yshift * (float)sin(angle);//coordinates of originating point
			yprime = (float)yshift * (float)cos(angle) - (float)xshift * (float)sin(angle);//in rotated axes
			xorig = xprime + width/2; yorig = yprime + height/2;//address of originating point
			x2 = (long)xorig; y2 = (long)yorig;//integer part of address
			dx = xorig - (float)x2; dy = yorig - (float)y2;//fractional part of address
			if ((y2 > 0) && (y2 < height-1) && (x2 > 0) && (x2 < width-1)) // if point is within image area
			{
				ReadOriginalLine(y2, LineA);//read two bracketing lines
				ReadOriginalLine(y2+1,LineB);
				valA = ((float)1.0-dx)*LineA[x2].red + dx*LineA[x2+1].red;//horizontal part of interpolation
				valB = ((float)1.0-dx)*LineB[x2].red + dx*LineB[x2+1].red;//(for each color channel)
				Result[x].red = ((float)1.0-dy)*valA + dy*valB;//vertical part of interpolation
				valA = ((float)1.0-dx)*LineA[x2].green + dx*LineA[x2+1].green;
				valB = ((float)1.0-dx)*LineB[x2].green + dx*LineB[x2+1].green;
				Result[x].green = ((float)1.0-dy)*valA + dy*valB;
				valA = ((float)1.0-dx)*LineA[x2].blue + dx*LineA[x2+1].blue;
				valB = ((float)1.0-dx)*LineB[x2].blue + dx*LineB[x2+1].blue;
				Result[x].blue = ((float)1.0-dy)*valA + dy*valB;
			}
			else // outside original image area
			{	Result[x].red = Result[x].green = Result[x].blue = 0;
			}
		}// for x
		WriteResultLine(y, Result);//write new values to final image
	}//for y
	
mainexit:	//clean up everything we allocated
	if (Result) DisposeAPointer(Result);
	if (LineA) DisposeAPointer(LineA);
	if (LineB) DisposeAPointer(LineB);

	return result;											//pass any errors back
}//MainUserEntry

