
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 1.5.4.1b Read Reference Image */

#include "PhotoshopShell.h"

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	rheight, rwidth;								//ref image dimensions
	long	mwidth;											//length of line to allocate
	long	x, y;											//loop counter to traverse image
	RGBPixel *line = NULL, *rline = NULL;					//memory buffers for lines to read, write

	GetOriginalDimensions(&width, &height);					//find out how big the image is
	GetRefDimensions(&rwidth, &rheight);					//find out how big the current ref image is

	mwidth = width; if (rwidth>mwidth) mwidth = rwidth;		//use whichever is larger for allocation

	line = CreateAPointer(width, sizeof(RGBPixel));			//Allocate memory for one row from each image
	rline = CreateAPointer(rwidth, sizeof(RGBPixel));

	if (rheight == height)									//they match - simple case
	{	for (y = 0; y < height; y++)
		{
			if (TestAbortSetProgressBar((float)y/(float)height))
			{
				result = userCanceledErr;
				goto mainexit;
			}
			ReadRefImageLine(y, rline); // read a line from ref image
			if (rwidth == width) WriteResultLine(y, rline); // if dimensions match...
			else if (rwidth > width) // otherwise center horizontally
			{	long dx = (rwidth - width)/2; // offset value
				for (x = 0; x < width; x++)
					line[x] = rline[x+dx];  // copy pixel by pixel
				WriteResultLine(y, line);	// and write result line
			}
			else // (width > rwidth)
			{
				long dx = (width - rwidth)/2; //offset value
				for (x = 0; x < width; x++)
				{	
					if ((x < dx) || (x >= rwidth+dx)) // if outside image area write black
						line[x].red = line[x].green = line[x].blue = 0;
					else line[x] = rline[x-dx];
				}
				WriteResultLine(y, line); // write result line
			}
		}//for y
	}//if
	else if (rheight > height)			// ref image is taller than current image
	{
		long dy = (rheight - height)/2; // center vertically
		for (y = 0; y < height; y++)	//line by line of current image
		{
			if (TestAbortSetProgressBar((float)y/(float)height))
			{
				result = userCanceledErr;
				goto mainexit;
			}
			ReadRefImageLine(y + dy, rline); // read ref image line
			if (rwidth == width) WriteResultLine(y, rline); // if widths match
			else if (rwidth > width) // otherwise, center it
			{	long dx = (rwidth - width)/2; // horizontal offset
				for (x = 0; x < width; x++)	//copy pixel by pixel
					line[x] = rline[x+dx];
				WriteResultLine(y, line);	//and write result line
			}
			else // (width > rwidth)
			{
				long dx = (width - rwidth)/2; // offset
				for (x = 0; x < width; x++)
				{
					if ((x < dx) || (x >= rwidth+dx)) // if outside area, write black
						line[x].red = line[x].green = line[x].blue = 0;
					else line[x] = rline[x-dx];
				}
				WriteResultLine(y, line); //write result line
			}
		}//for y
	}//else if
	else // (height > rheight)
	{
		long dy = (height - rheight)/2;	//center vertically
		for (y = 0; y < height; y++)//line by line of current image
		{
			if (TestAbortSetProgressBar((float)y/(float)height))
			{
				result = userCanceledErr;
				goto mainexit;
			}
			if ((y < dy) || (y >= rheight+dy)) // if outside area write black
			{	for (x = 0; x < width; x++)
					line[x].red = line[x].green = line[x].blue = 0;
				WriteResultLine(y, line);
			}
			else
			{	ReadRefImageLine(y - dy, rline); // read line from ref image
				if (rwidth == width) WriteResultLine(y, rline);//if dimensions match
				else if (rwidth > width)
				{
					long dx = (rwidth - width)/2;//horizontal offset
					for (x = 0; x < width; x++)
						line[x] = rline[x+dx];//copy pixel by pixel
					WriteResultLine(y, line);
				}
				else // (width > rwidth)
				{	long dx = (width - rwidth)/2;
					for (x = 0; x < width; x++)
					{	if ((x < dx) || (x >= rwidth+dx))
							line[x].red = line[x].green = line[x].blue = 0;
						else line[x] = rline[x-dx];
					}
					WriteResultLine(y, line);//write result back
				}
			}
		}//for y
	}//else


mainexit:													//clean up everything we allocated
	if (line)
		DisposeAPointer(line);
	if (rline)
		DisposeAPointer(rline);

	return result;											//pass any errors back
}//MainUserEntry

