
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 1.5.4.3 Subtract Reference Image */

#include "PhotoshopShell.h"

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	rheight, rwidth;								//and ref image
	long	x, y;											//loop counter to traverse image
	float	min, max, temp;
	Boolean	Adding = FALSE; 								//true to add, false to subtract
	RGBPixel *line = NULL, *rline = NULL;					//memory buffers

	GetOriginalDimensions(&width, &height);					//find out how big the image is
	GetRefDimensions(&rwidth, &rheight);					//find out how big the current ref image is

	if ((height != rheight) || (width != rwidth))
	{
		ErrorMessage("Dimensions of image and reference image do not match.");
		return (userCanceledErr);
	}
	//or, could process a part of the image, or could scale ref image to match current image...

	line = CreateAPointer(width, sizeof(RGBPixel));			//Allocate memory for one row from each image
	rline = CreateAPointer(width, sizeof(RGBPixel));

	min = 512; max = -256;
	// first pass - find limits (for all three channels)
	for (y = 0; y < height; y++)//line by line
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadRefImageLine(y, rline);//read one line from each image
		ReadOriginalLine(y, line);
		for (x = 0; x < width; x++)//pixel by pixel
		{	if (!Adding)
			{	temp = line[x].red - rline[x].red;//subtract each color channel
				if (temp < min) min = temp;	//find minimum and maximum limits for autoscaling
				if (temp > max) max = temp;
				temp = line[x].green - rline[x].green;
				if (temp < min) min = temp;
				if (temp > max) max = temp;
				temp = line[x].blue - rline[x].blue;
				if (temp < min) min = temp;
				if (temp > max) max = temp;
			}
			else // adding
			{	temp = line[x].red + rline[x].red;
				if (temp < min) min = temp;
				if (temp > max) max = temp;
				temp = line[x].green + rline[x].green;
				if (temp < min) min = temp;
				if (temp > max) max = temp;
				temp = line[x].blue + rline[x].blue;
				if (temp < min) min = temp;
				if (temp > max) max = temp;
			}
		}
	}
	// second pass - perform math
	for (y = 0; y < height; y++)//same as first pass except now results are saved
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadRefImageLine(y, rline);
		ReadOriginalLine(y, line);
		for (x = 0; x < width; x++)
		{	if (!Adding)
			{	temp = line[x].red - rline[x].red;// get difference
				line[x].red = (float)255.0 * (temp - min) / (max - min);//and scale to fit 0..255 range
				temp = line[x].green - rline[x].green;
				line[x].green = (float)255.0 * (temp - min) / (max - min);
				temp = line[x].blue - rline[x].blue;
				line[x].blue = (float)255.0 * (temp - min) / (max - min);
			}
			else // adding
			{	temp = line[x].red + rline[x].red;
				line[x].red = (float)255.0 * (temp - min) / (max - min);
				temp = line[x].green + rline[x].green;
				line[x].green = (float)255.0 * (temp - min) / (max - min);
				temp = line[x].blue + rline[x].blue;
				line[x].blue = (float)255.0 * (temp - min) / (max - min);
			}
		}
		WriteResultLine(y, line);
	}
mainexit:													//clean up everything we allocated
	if (line)
		DisposeAPointer(line);
	if (rline)
		DisposeAPointer(rline);

	return result;											//pass any errors back
}//MainUserEntry

