
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 1.5.4.4 Apply Function */

#include "PhotoshopShell.h"

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;							//accumulate any errors that happened
	long	fref;									//file ID to read instruction
	long	height, width;							//pixel dimensions of image
	long	x, y;									//loop counters to traverse image
	float	bright;									//0..255 value
	float	value;									//0..1 value
	long	function;	//read from file 1=log, 2=exp, 3=sqr, 4=sqrt, 5=reverse, 6=1/v
	RGBPixel *line = NULL;							//memory buffer for line of pixels
	
	GetOriginalDimensions(&width, &height);			//find out how big the image is
	line = CreateAPointer(width, sizeof(RGBPixel));	//allocate memory for one row

	if (OpenTextFileToRead("C:\\function.txt", &fref) != noErr) // select function to perform
	{	result = fnfErr;
		goto mainexit;
	}
	else
	{	result = ReadANumber(fref, &value); // reads as a float
		CloseTextFile(fref);
		if (result != noErr)
			goto mainexit;
	}
	function=(long)(value+0.5);

	if ((function < 1) || (function > 6)) // need a legal value
	{	result = fnfErr;
		goto mainexit;
	}

	for (y = 0; y < height; y++)//line by line
	{
		if (TestAbortSetProgressBar((float)y/(float)height*2))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadOriginalLine(y, line);				//read a line
		
		for (x = 0; x < width; x++)// and pixel by pixel
		{
			float A,B;	//keep color info unchanged
			bright = (line[x].red + line[x].green +line[x].blue)/3; // brightness (average)
			A = (line[x].red - line[x].green);// spherical LAB color coordinates
			B = (line[x].red + line[x].green)/2 - line[x].blue;
			switch (function)
			{
				case 1:	// log
					value = (float)(log(bright+1)/log(256));//+1 to avoid log(0)
					bright = 256 * value - 1;
				break;
				case 2:	// exp
					value = bright/255;
					value = ((float)exp(value)-1)/(float)(exp(1)-1);
					bright = 255 * value;
				break;
				case 3: // square
					value = bright/255;
					value = value*value;
					bright = 255 * value;
				break;
				case 4: // square root
					value = bright/255;
					value = (float)sqrt(value);
					bright = 255*value;
				break;
				case 5: // reverse (sometime called 'invert')
					value = 255-bright;
					bright = value;
				break;
				case 6: // inverse (1/value as opposed to 255-value)
					value = 256/(bright+1); // +1 to avoid div by zero
					bright = value - 1;
				break;

			}
			//combine new brightness value with prior color info and recalculate RGB	
			line[x].red =   bright + A/2 + B/3;
			line[x].green = bright - A/2 + B/3;
			line[x].blue =  bright - 2 * B/3;
		}//for x
		WriteResultLine(y, line);				//write changed line back
	}//for y

mainexit:										//clean up everything we allocated
	if (line)
		DisposeAPointer(line);

	return result;								//pass any errors back
}//MainUserEntry

