
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 2.1.10.1 3x3 Filter */

#include "PhotoshopShell.h"

typedef struct	// define a struct to hold pixel LAB values 
{	float L, A, B;
}LABPixel;

typedef	RGBPixel *RGBLinePtr;	//pointers to lines of pixels with RGB values
typedef LABPixel *LABLinePtr;	//same for LAB values


void ConvertLineRGBtoLAB(long width, RGBPixel *RGBline, LABPixel *LABline);
void ConvertLineRGBtoLAB(long width, RGBPixel *RGBline, LABPixel *LABline)
{	
	long	x; // pixel address across each line
	for (x = 0; x < width; x++)
	{		
		// convert RGB to spherical LAB values
		LABline[x].L = (RGBline[x].red + RGBline[x].green + RGBline[x].blue)/3;
		LABline[x].A = (RGBline[x].red - RGBline[x].green);
		LABline[x].B = (RGBline[x].red + RGBline[x].green)/2 - RGBline[x].blue;
	}
}

void ConvertLineLABtoRGB(long width, LABPixel *LABline, RGBPixel *RGBline);
void ConvertLineLABtoRGB(long width, LABPixel *LABline, RGBPixel *RGBline)
{	
	long	x;
	for (x = 0; x < width; x++)
	{	// convert from spherical LAB values to RGB
		RGBline[x].red =   LABline[x].L + LABline[x].A/2 + LABline[x].B/3;
		RGBline[x].green = LABline[x].L - LABline[x].A/2 + LABline[x].B/3;
		RGBline[x].blue =  LABline[x].L - 2 * LABline[x].B/3;
	}
}

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	ErrType theerr = noErr;
	long	height, width;									//pixel dimensions of image
	long	x, y;											//loop counters to traverse image
	long	i, j;											//loop counters for kernel
	float	kernel[3][3];									//3x3 weights
	float	sum, divisor, offset;							//calculating the new pixel values
	float	pixel[3][3];									//pixel values used for summation
	RGBLinePtr RGBline[3];									//memory buffer - 3 lines of RGB values
	LABLinePtr LABline[3];									//and three lines of LAB values
	RGBPixel *ResultRGB;									//result to write back final image
	LABPixel *ResultLAB;									//final result in LAB values
	long	fref;											//file ref number for filter values

	GetOriginalDimensions(&width, &height);					//find out how big the image is

	for (i = 0; i < 3; i++)									//initialize the pointer to null
	{														//not essential but good form
		RGBline[i] = NULL;
		LABline[i] = NULL;
	}
	ResultRGB = NULL;
	ResultLAB = NULL;
	
	for (i = 0; i < 3; i++)									//Allocate memory for 3 rows
	{		
		RGBline[i] = CreateAPointer(width, sizeof(RGBPixel));
		LABline[i] = CreateAPointer(width, sizeof(LABPixel));	
	}
	ResultRGB = CreateAPointer(width, sizeof(RGBPixel));	//and for results
	ResultLAB = CreateAPointer(width, sizeof(LABPixel));

	if (OpenTextFileToRead("C:\\filter.txt", &fref) != noErr) //find the kernel file
	{	
		result = fnfErr;
		goto mainexit;
	}
	else													//read the nine values for the kernel
	{	
		for (i = 0; i < 3; i++)
			for (j = 0; j < 3; j++) 
				theerr = ReadANumber(fref, &kernel[i][j]);
		ReadANumber(fref, &divisor);						//and the divisor and offset
		ReadANumber(fref, &offset);
		CloseTextFile(fref);
	}
/*	// default sharpening kernel for testing, if the file reading code is commented out
	kernel[0][0] = -1.0; kernel[0][1] = -1.0; kernel[0][2] = -1.0;
	kernel[1][0] = -1.0; kernel[1][1] = +9.0; kernel[1][2] = -1.0;
	kernel[2][0] = -1.0; kernel[2][1] = -1.0; kernel[2][2] = -1.0;
	divisor = 1.0; offset = 0.0;
*/

	// 1. Traverse image one line at a time y=0..height-1
	// 2. read lines y-1, y, y+1
	// 3. convert lines to LAB space
	// 4. traverse lines horizontally x=0..width-1
	// 5. copy out each pixel 3x3 neighborhood (L value only)
	// 6. apply convolution kernel to pixel 3x3 neighborhood
	// 7. store resultline[x]
	// 8. convert resultline back to RGB (using preserved A and B values)
	// 9. write line y back to image
	
	for (y = 0; y < height; y++)
	{	
		if (TestAbortSetProgressBar((float)y/(float)height))
		{	//See if user canceled and update progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		if (y > 0)	ReadOriginalLine(y-1, RGBline[0]);			//read line above
		else		ReadOriginalLine(0, RGBline[0]);			//edge protection at top
		if (y < height-1)	ReadOriginalLine(y+1, RGBline[2]);	//line below
		else				ReadOriginalLine(height-1, RGBline[2]);//edge protection at bottom
		ReadOriginalLine(y, RGBline[1]);						//read the current line

		for (i = 0; i < 3; i ++)
			ConvertLineRGBtoLAB(width, RGBline[i], LABline[i]);	//use LAB values for calculating
			
		for (x = 0; x < width; x++)								//process each pixel along the line
		{	
			for (i = 0; i < 3; i++)								//read neighbors left to right
			{	
				if (x>0) pixel[i][0] = LABline[i][x-1].L;		
				else	 pixel[i][0] = LABline[i][0].L;			//edge protection at left
				pixel[i][1] =			   LABline[i][x].L;
				if (x < width-1) pixel[i][2] = LABline[i][x+1].L;
				else			 pixel[i][2] = LABline[i][width-1].L;// and at right
			}
			// apply the convolution kernel to the pixels
			sum = 0;
			for (i = 0; i < 3; i++)
				for (j = 0; j < 3; j++)
					sum += kernel[i][j] * pixel[i][j];
			sum = (sum / divisor) + offset;
			if (sum < 0) sum = 0;								//limit results to 0..255
			if (sum > 255) sum = 255;
			ResultLAB[x].L = sum;								//new L value
			ResultLAB[x].A = LABline[1][x].A;					//keep original color values
			ResultLAB[x].B = LABline[1][x].B;
		}//for x
		ConvertLineLABtoRGB(width, ResultLAB, ResultRGB);		//convert back to RGB
		WriteResultLine(y, ResultRGB);							//and write back the result
	}//for y
	
mainexit:	//clean up everything we allocated
	for (i = 0; i < 3; i++)
	{	
		if (RGBline[i]) DisposeAPointer(RGBline[i]);
		if (LABline[i]) DisposeAPointer(LABline[i]);
	}
	if (ResultRGB) DisposeAPointer(ResultRGB);
	if (ResultLAB) DisposeAPointer(ResultLAB);

	return result;											//pass any errors back
}//MainUserEntry

