
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 2.1.10.13 Sobel */

#include "PhotoshopShell.h"

typedef struct
{	float L, A, B;
}LABPixel;

typedef	RGBPixel *RGBLinePtr;
typedef LABPixel *LABLinePtr;


void ConvertLineRGBtoLAB(long width, RGBPixel *RGBline, LABPixel *LABline);
void ConvertLineRGBtoLAB(long width, RGBPixel *RGBline, LABPixel *LABline)
{	long	x;
	for (x = 0; x < width; x++)
	{	// convert to spherical LAB values
		LABline[x].L = (RGBline[x].red + RGBline[x].green + RGBline[x].blue)/3;
		LABline[x].A = (RGBline[x].red - RGBline[x].green);
		LABline[x].B = (RGBline[x].red + RGBline[x].green)/2 - RGBline[x].blue;
	}
}

void ConvertLineLABtoRGB(long width, LABPixel *LABline, RGBPixel *RGBline);
void ConvertLineLABtoRGB(long width, LABPixel *LABline, RGBPixel *RGBline)
{	long	x;
	for (x = 0; x < width; x++)
	{	// convert from spherical LAB values
		RGBline[x].red =   LABline[x].L + LABline[x].A/2 + LABline[x].B/3;
		RGBline[x].green = LABline[x].L - LABline[x].A/2 + LABline[x].B/3;
		RGBline[x].blue =  LABline[x].L - 2 * LABline[x].B/3;
	}
}

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	ErrType theerr = noErr;
	long	height, width;									//pixel dimensions of image
	long	x, y;											//loop counters to traverse image
	long	i, j, h, k;
	float	vert, horz, bright, angle;
	float	pixel[3][3];								//pixel values
	Boolean	magnitude = TRUE;
	RGBLinePtr RGBline[3];								//memory buffer - 5 lines
	LABLinePtr LABline[3];
	RGBPixel *ResultRGB;
	LABPixel *ResultLAB;

	GetOriginalDimensions(&width, &height);					//find out how big the image is

	for (i = 0; i < 3; i++)
	{	RGBline[i] = NULL;
		LABline[i] = NULL;
	}
	ResultRGB = NULL;
	ResultLAB = NULL;
	for (i = 0; i < 3; i++)
	{	RGBline[i] = CreateAPointer(width, sizeof(RGBPixel));
		LABline[i] = CreateAPointer(width, sizeof(LABPixel));	//Allocate memory for N rows
	}
	ResultRGB = CreateAPointer(width, sizeof(RGBPixel));
	ResultLAB = CreateAPointer(width, sizeof(LABPixel));

	for (y = 0; y < height; y++)
	{	if (TestAbortSetProgressBar((float)y/(float)height))
		{	//See if user canceled and update progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		for (i = 0; i < 3; i++)
		{	h = y + i - 1;
			if (h<0) ReadOriginalLine(0, RGBline[i]); // boundary protection
			else if (h>=height) ReadOriginalLine(height-1, RGBline[i]);
			else ReadOriginalLine(h, RGBline[i]);
		}
		for (i = 0; i < 3; i ++)
			ConvertLineRGBtoLAB(width, RGBline[i], LABline[i]);
		
		for (x = 0; x < width; x++)
		{	for (i = 0; i < 3; i++)
				for (j = 0; j < 3; j++)
				{	k = x + j - 1;
					if (k<0) pixel[i][j] = LABline[i][0].L;
					else if (k>=width) pixel[i][j] = LABline[i][width-1].L;
					else pixel[i][j] = LABline[i][k].L;
				}
			// apply the convolution kernels to the pixels
			vert = 0; horz = 0;
			for (i = 0; i < 3; i++)
			{	vert += pixel[0][i] - pixel[2][i];
				horz += pixel[i][0] - pixel[i][2];
			}
			if (magnitude)
			{	bright = (float)sqrt(vert*vert+horz*horz);
			}
			else
			{	angle = (float)atan(vert/horz);
				bright = 64+(float)(127.5/3.1415926535897) * angle;
				if (horz<0) bright += 127.5;
			}
			if (bright<0) bright = 0; if (bright>255) bright = 255;
			ResultLAB[x].L = bright;
			ResultLAB[x].A = LABline[1][x].A;
			ResultLAB[x].B = LABline[1][x].B;
		}//for x
		ConvertLineLABtoRGB(width, ResultLAB, ResultRGB);
		WriteResultLine(y, ResultRGB);
	}//for y
	
mainexit:	//clean up everything we allocated
	for (i = 0; i < 3; i++)
	{	if (RGBline[i]) DisposeAPointer(RGBline[i]);
		if (LABline[i]) DisposeAPointer(LABline[i]);
	}
	if (ResultRGB) DisposeAPointer(ResultRGB);
	if (ResultLAB) DisposeAPointer(ResultLAB);

	return result;											//pass any errors back
}//MainUserEntry

