
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 2.1.10.4 Autoscale Filter */

#include "PhotoshopShell.h"

typedef struct	// holds spherical LAB values for each pixel
{	float L, A, B;
}LABPixel;

typedef	RGBPixel *RGBLinePtr; // pointer for RGB pixel values
typedef LABPixel *LABLinePtr; // and for LAB values


void ConvertLineRGBtoLAB(long width, RGBPixel *RGBline, LABPixel *LABline);
void ConvertLineRGBtoLAB(long width, RGBPixel *RGBline, LABPixel *LABline)
{	
	long	x; // pixel addresses across each line
	for (x = 0; x < width; x++)
	{	// convert RGB to spherical LAB values
		LABline[x].L = (RGBline[x].red + RGBline[x].green + RGBline[x].blue)/3;
		LABline[x].A = (RGBline[x].red - RGBline[x].green);
		LABline[x].B = (RGBline[x].red + RGBline[x].green)/2 - RGBline[x].blue;
	}
}

void ConvertLineLABtoRGB(long width, LABPixel *LABline, RGBPixel *RGBline);
void ConvertLineLABtoRGB(long width, LABPixel *LABline, RGBPixel *RGBline)
{
	long	x;
	for (x = 0; x < width; x++)
	{	// convert from spherical LAB values to RGB
		RGBline[x].red =   LABline[x].L + LABline[x].A/2 + LABline[x].B/3;
		RGBline[x].green = LABline[x].L - LABline[x].A/2 + LABline[x].B/3;
		RGBline[x].blue =  LABline[x].L - 2 * LABline[x].B/3;
	}
}
#define RAD 2 // neighborhood size = 5x5 (central pixel plus 2 in all directions)
#define size 2*RAD+1 // kernel size

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happen
	ErrType theerr = noErr;
	long	height, width;									//pixel dimensions of image
	long	x, y;											//loop counters to traverse image
	long	i, j, h, k;										//loop counters for kernel
	float	min, max;										//used to autoscale results
	float	kernel[size][size];								//5x5 weights
	float	sum;											//sum of weights times pixel values
	float	pixel[size][size];								//pixel values
	RGBLinePtr RGBline[size];								//memory buffer - 5 lines
	LABLinePtr LABline[size];								//converted RGB->LAB pixels
	RGBPixel *ResultRGB;									//result to write back to image
	LABPixel *ResultLAB;									//final result in LAB values
	long	fref;											//file ref number for filter values

	GetOriginalDimensions(&width, &height);					//find out how big the image is

	for (i = 0; i < size; i++)
	{	RGBline[i] = NULL;									//initialize pointers to NULL
		LABline[i] = NULL;
	}
	ResultRGB = NULL;
	ResultLAB = NULL;
	for (i = 0; i < size; i++)
	{	RGBline[i] = CreateAPointer(width, sizeof(RGBPixel));
		LABline[i] = CreateAPointer(width, sizeof(LABPixel));	//Allocate memory for multiple rows
	}
	ResultRGB = CreateAPointer(width, sizeof(RGBPixel));	//and for results
	ResultLAB = CreateAPointer(width, sizeof(LABPixel));

	if (OpenTextFileToRead("C:\\filter5.txt", &fref) != noErr) //fine the kernel file
	{	
		result = fnfErr;
		goto mainexit;
	}
	else
	{	for (i = 0; i < size; i++)
			for (j = 0; j < size; j++) 
				theerr = ReadANumber(fref, &kernel[i][j]);
		CloseTextFile(fref);
		if (theerr != noErr)
			{	
				result = theerr;
				goto mainexit;
			}	
	}
	
	// autoscaling, so do not need a divisor and offset as in problem 2.1.10.1
	 
/*	// default sharpening kernel for testing (comment out the file reading above)
	kernel[0][0] = -1.0; kernel[0][1] = -1.0; kernel[0][2] = -1.0; kernel[0][3] = -1.0; kernel[0][4] = -1.0;
	kernel[1][0] = -1.0; kernel[1][1] = +1.0; kernel[1][2] = +2.0; kernel[1][3] = +1.0; kernel[1][4] = -1.0;
	kernel[2][0] = -1.0; kernel[2][1] = +2.0; kernel[2][2] = +4.0; kernel[2][3] = +2.0; kernel[2][4] = -1.0;
	kernel[3][0] = -1.0; kernel[3][1] = +1.0; kernel[3][2] = +2.0; kernel[3][3] = +1.0; kernel[3][4] = -1.0;
	kernel[4][0] = -1.0; kernel[4][1] = -1.0; kernel[4][2] = -1.0; kernel[4][3] = -1.0; kernel[4][4] = -1.0;
*/
	min = (float)1e20; max = 0; // initialize min and max
	for (y = 0; y < height; y++) // first pass - find min & max result values
	{	
		if (TestAbortSetProgressBar((float)y/(float)height))
		{	//See if user canceled and update progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		for (i = 0; i < size; i++)
		{	
			h = y + i - RAD; // offset to read neighboring lines
			if (h<0) ReadOriginalLine(0, RGBline[i]); // boundary protection at top
			else if (h>=height) ReadOriginalLine(height-1, RGBline[i]); // and bottom
			else ReadOriginalLine(h, RGBline[i]);
		}
		for (i = 0; i < size; i ++)
			ConvertLineRGBtoLAB(width, RGBline[i], LABline[i]); // change RGB -> spherical LAB
		
		for (x = 0; x < width; x++)
		{	
			for (i = 0; i < size; i++) // traverse neighborhood pixels
				for (j = 0; j < size; j++)
				{	
					k = x + j - RAD; // offset to left/right neighbors
					if (k<0) pixel[i][j] = LABline[i][0].L; //boundary protection at left
					else if (k>=width) pixel[i][j] = LABline[i][width-1].L;// and right
					else pixel[i][j] = LABline[i][k].L;
				}
			// apply the convolution kernel to the pixels
			sum = 0;
			for (i = 0; i < size; i++)
				for (j = 0; j < size; j++)
					sum += kernel[i][j] * pixel[i][j];
			if (sum < min) min = sum; // test and find max/min limits
			if (sum > max) max = sum;
		}//for x
	}//for y

	//now repeat it all again, autoscale the results, and write the results back
	for (y = 0; y < height; y++)
	{	
		if (TestAbortSetProgressBar((float)y/(float)height))
		{	//See if user canceled and update progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		for (i = 0; i < size; i++)
		{	
			h = y + i - RAD;
			if (h<0) ReadOriginalLine(0, RGBline[i]); // boundary protection
			else if (h>=height) ReadOriginalLine(height-1, RGBline[i]);
			else ReadOriginalLine(h, RGBline[i]);
		}
		for (i = 0; i < size; i ++)
			ConvertLineRGBtoLAB(width, RGBline[i], LABline[i]);
		
		for (x = 0; x < width; x++)
		{	
			for (i = 0; i < size; i++)
				for (j = 0; j < size; j++)
				{	
					k = x + j - RAD;
					if (k<0) pixel[i][j] = LABline[i][0].L;
					else if (k>=width) pixel[i][j] = LABline[i][width-1].L;
					else pixel[i][j] = LABline[i][k].L;
				}
			// apply the convolution kernel to the pixels
			sum = 0;
			for (i = 0; i < size; i++)
				for (j = 0; j < size; j++)
					sum += kernel[i][j] * pixel[i][j];
			sum = 255*(sum-min)/(max-min); // but this time rescale the results to 0..255
			ResultLAB[x].L = sum;
			ResultLAB[x].A = LABline[RAD][x].A; // copy over the original color information
			ResultLAB[x].B = LABline[RAD][x].B;
		}//for x
		ConvertLineLABtoRGB(width, ResultLAB, ResultRGB);// and convert back to RGB
		WriteResultLine(y, ResultRGB);// then rewrite results to image
	}//for y
	
mainexit:	//clean up everything we allocated
	for (i = 0; i < size; i++)
	{	if (RGBline[i]) DisposeAPointer(RGBline[i]);
		if (LABline[i]) DisposeAPointer(LABline[i]);
	}
	if (ResultRGB) DisposeAPointer(ResultRGB);
	if (ResultLAB) DisposeAPointer(ResultLAB);

	return result;											//pass any errors back
}//MainUserEntry

