
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 2.2.7.1 Median Filter */

#include "PhotoshopShell.h"

typedef struct // struct to hold spherical LAB values for a pixel
{	float L, A, B;
}LABPixel;

typedef	RGBPixel *RGBLinePtr; // pointer to hold a line of RGB pixel values
typedef LABPixel *LABLinePtr; // and a line of spherical LAB values

void ConvertLineRGBtoLAB(long width, RGBPixel *RGBline, LABPixel *LABline);
void ConvertLineRGBtoLAB(long width, RGBPixel *RGBline, LABPixel *LABline)
{
	long	x;
	for (x = 0; x < width; x++)
	{	// convert RGB to spherical LAB values
		LABline[x].L = (RGBline[x].red + RGBline[x].green + RGBline[x].blue)/3;
		LABline[x].A = (RGBline[x].red - RGBline[x].green);
		LABline[x].B = (RGBline[x].red + RGBline[x].green)/2 - RGBline[x].blue;
	}
}

void ConvertLineLABtoRGB(long width, LABPixel *LABline, RGBPixel *RGBline);
void ConvertLineLABtoRGB(long width, LABPixel *LABline, RGBPixel *RGBline)
{
	long	x;
	for (x = 0; x < width; x++)
	{	// convert from spherical LAB values to RGB
		RGBline[x].red =   LABline[x].L + LABline[x].A/2 + LABline[x].B/3;
		RGBline[x].green = LABline[x].L - LABline[x].A/2 + LABline[x].B/3;
		RGBline[x].blue =  LABline[x].L - 2 * LABline[x].B/3;
	}
}

//sorting routine sorts values in array1 (brightness values) but also rearranges the values
//in arrays 2 and 3 (color values) so they are carried along.
void SortArrays(float *array1, float *array2, float *array3, long N);
void SortArrays(float *array1, float *array2, float *array3, long N)
{	//sort can handle any size array
	Boolean	done;
	long	jump, i, j;
	float	temp;

	jump = N;
	while (jump > 0)
	{	jump /= 2;
		do
		{	done = TRUE;
			for (j = 0; j < N-jump; j++)
			{	i = j + jump;
				if (array1[j] > array1[i])
				{	//sort on array 1 but carry 2, 3 values along
					temp = array1[j]; array1[j] = array1[i]; array1[i] = temp;
					temp = array2[j]; array2[j] = array2[i]; array2[i] = temp;
					temp = array3[j]; array3[j] = array3[i]; array3[i] = temp;
					done = FALSE;
				}
			}
		} while (!done);
	}
}

#define RAD 3 // max neighborhood size = 7x7 (central pixel plus 3 in each direction)
#define size 2*RAD+1 // neighborhood size
#define array 37 //large enough for 7 wide circular nborhood
///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	x, y;											//loop counters to traverse image
	long	i, j, h, k;										//loop counters for neighborhood
	long	medsize, arraysize, medval;										//radius = 1, 2 or 3
	float	pixelL[size][size], pixelA[size][size], pixelB[size][size];		//pixel values in LAB
	float SortL[array], SortA[array], SortB[array];			//arrays for sorting
	RGBLinePtr RGBline[size];								//memory buffer - 7 lines of pixels
	LABLinePtr LABline[size];								//same for converted LAB values
	RGBPixel *ResultRGB;									//line of final results
	LABPixel *ResultLAB;									//line of intermediate LAB pixel results

	GetOriginalDimensions(&width, &height);					//find out how big the image is

	for (i = 0; i < size; i++)								//initialize pointers to null (easier cleanup)
	{
		RGBline[i] = NULL;
		LABline[i] = NULL;
	}
	ResultRGB = NULL;
	ResultLAB = NULL;
	
	for (i = 0; i < size; i++)
	{
		RGBline[i] = CreateAPointer(width, sizeof(RGBPixel));
		LABline[i] = CreateAPointer(width, sizeof(LABPixel));	//Allocate memory for mulitple rows
	}
	ResultRGB = CreateAPointer(width, sizeof(RGBPixel));	//and for results
	ResultLAB = CreateAPointer(width, sizeof(LABPixel));

	medsize = 2; // or read neighborhood radius from a text file, can be radius = 1, 2 or 3
	switch (medsize)
	{	//define the array size and the position of the median as a function of the radius
		case 1: arraysize = 9;  medval = 4;  break;
		case 2: arraysize = 21; medval = 10; break;
		case 3: arraysize = 37; medval = 18; break;
	}

	for (y = 0; y < height; y++)// scan image line by line
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{	//See if user canceled and update progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		for (i = 0; i < size; i++)//read multiple lines
		{
			h = y + i - RAD;// vertical offset
			if (h<0) ReadOriginalLine(0, RGBline[i]); // boundary protection at top
			else if (h>=height) ReadOriginalLine(height-1, RGBline[i]);//and bottom
			else ReadOriginalLine(h, RGBline[i]);
		}
		for (i = 0; i < size; i ++)
			ConvertLineRGBtoLAB(width, RGBline[i], LABline[i]);//convert RGB to LAB
		
		for (x = 0; x < width; x++) // scan each line pixel by pixel
		{
			for (i = 0; i < size; i++)// fill arrays with pixels in neighborhood
				for (j = 0; j < size; j++)
				{	k = x + j - RAD;//horizontal offset
					if (k<0) //boundary protection at left
					{	pixelL[i][j] = LABline[i][0].L;
						pixelA[i][j] = LABline[i][0].A;
						pixelB[i][j] = LABline[i][0].B;
					}
					else if (k>=width) //and right
					{	pixelL[i][j] = LABline[i][width-1].L;
						pixelA[i][j] = LABline[i][width-1].A;
						pixelB[i][j] = LABline[i][width-1].B;
					}
					else 
					{	pixelL[i][j] = LABline[i][k].L;
						pixelA[i][j] = LABline[i][k].A;
						pixelB[i][j] = LABline[i][k].B;
					}
				}
			//Copy pixel values into arrays, working outward from the center
			//This could be reduced to a loop structure but is made explicit for clarity
			SortL[0] = pixelL[RAD  ][RAD  ]; SortA[0] = pixelA[RAD  ][RAD  ]; SortB[0] = pixelB[RAD  ][RAD  ];
			SortL[1] = pixelL[RAD-1][RAD  ]; SortA[1] = pixelA[RAD-1][RAD  ]; SortB[1] = pixelB[RAD-1][RAD  ];
			SortL[2] = pixelL[RAD-1][RAD-1]; SortA[2] = pixelA[RAD-1][RAD-1]; SortB[2] = pixelB[RAD-1][RAD-1];
			SortL[3] = pixelL[RAD  ][RAD-1]; SortA[3] = pixelA[RAD  ][RAD-1]; SortB[3] = pixelB[RAD  ][RAD-1];
			SortL[4] = pixelL[RAD+1][RAD-1]; SortA[4] = pixelA[RAD+1][RAD-1]; SortB[4] = pixelB[RAD+1][RAD-1];
			SortL[5] = pixelL[RAD+1][RAD  ]; SortA[5] = pixelA[RAD+1][RAD  ]; SortB[5] = pixelB[RAD+1][RAD  ];
			SortL[6] = pixelL[RAD+1][RAD+1]; SortA[6] = pixelA[RAD+1][RAD+1]; SortB[6] = pixelB[RAD+1][RAD+1];
			SortL[7] = pixelL[RAD  ][RAD+1]; SortA[7] = pixelA[RAD  ][RAD+1]; SortB[7] = pixelB[RAD  ][RAD+1];
			SortL[8] = pixelL[RAD-1][RAD+1]; SortA[8] = pixelA[RAD-1][RAD+1]; SortB[8] = pixelB[RAD-1][RAD+1];
			//The first 9 values are the 3x3 neighborhood
			SortL[9] =  pixelL[RAD-2][RAD  ]; SortA[0] = pixelA[RAD-2][RAD  ]; SortB[0] = pixelB[RAD-2][RAD  ];
			SortL[10] = pixelL[RAD-2][RAD-1]; SortA[1] = pixelA[RAD-2][RAD-1]; SortB[1] = pixelB[RAD-2][RAD-1];
			SortL[11] = pixelL[RAD-1][RAD-2]; SortA[2] = pixelA[RAD-1][RAD-2]; SortB[2] = pixelB[RAD-1][RAD-2];
			SortL[12] = pixelL[RAD  ][RAD-2]; SortA[3] = pixelA[RAD  ][RAD-2]; SortB[3] = pixelB[RAD  ][RAD-2];
			SortL[13] = pixelL[RAD+1][RAD-2]; SortA[4] = pixelA[RAD+1][RAD-2]; SortB[4] = pixelB[RAD+1][RAD-2];
			SortL[14] = pixelL[RAD+2][RAD-1]; SortA[5] = pixelA[RAD+2][RAD-1]; SortB[5] = pixelB[RAD+2][RAD-1];
			SortL[15] = pixelL[RAD+2][RAD  ]; SortA[6] = pixelA[RAD+2][RAD  ]; SortB[6] = pixelB[RAD+2][RAD  ];
			SortL[16] = pixelL[RAD+2][RAD+1]; SortA[7] = pixelA[RAD+2][RAD+1]; SortB[7] = pixelB[RAD+2][RAD+1];
			SortL[17] = pixelL[RAD+1][RAD+2]; SortA[8] = pixelA[RAD+1][RAD+2]; SortB[8] = pixelB[RAD+1][RAD+2];
			SortL[18] = pixelL[RAD  ][RAD+2]; SortA[0] = pixelA[RAD  ][RAD+2]; SortB[0] = pixelB[RAD  ][RAD+2];
			SortL[19] = pixelL[RAD-1][RAD+2]; SortA[1] = pixelA[RAD-1][RAD+2]; SortB[1] = pixelB[RAD-1][RAD+2];
			SortL[20] = pixelL[RAD-2][RAD+1]; SortA[2] = pixelA[RAD-2][RAD+1]; SortB[2] = pixelB[RAD-2][RAD+1];
			//This completes the 5-wide circular neighborhood
			SortL[21] = pixelL[RAD-2][RAD-2]; SortA[3] = pixelA[RAD-2][RAD-2]; SortB[3] = pixelB[RAD-2][RAD-2];
			SortL[22] = pixelL[RAD-2][RAD+2]; SortA[4] = pixelA[RAD-2][RAD+2]; SortB[4] = pixelB[RAD-2][RAD+2];
			SortL[23] = pixelL[RAD+2][RAD+2]; SortA[5] = pixelA[RAD+2][RAD+2]; SortB[5] = pixelB[RAD+2][RAD+2];
			SortL[24] = pixelL[RAD+2][RAD-2]; SortA[6] = pixelA[RAD+2][RAD-2]; SortB[6] = pixelB[RAD+2][RAD-2];
			SortL[25] = pixelL[RAD-3][RAD-1]; SortA[7] = pixelA[RAD-3][RAD-1]; SortB[7] = pixelB[RAD-3][RAD-1];
			SortL[26] = pixelL[RAD-3][RAD  ]; SortA[8] = pixelA[RAD-3][RAD  ]; SortB[8] = pixelB[RAD-3][RAD  ];
			SortL[27] = pixelL[RAD-3][RAD+1]; SortA[0] = pixelA[RAD-3][RAD+1]; SortB[0] = pixelB[RAD-3][RAD+1];
			SortL[28] = pixelL[RAD+3][RAD-1]; SortA[1] = pixelA[RAD+3][RAD-1]; SortB[1] = pixelB[RAD+3][RAD-1];
			SortL[29] = pixelL[RAD+3][RAD  ]; SortA[2] = pixelA[RAD+3][RAD  ]; SortB[2] = pixelB[RAD+3][RAD  ];
			SortL[30] = pixelL[RAD+3][RAD+1]; SortA[3] = pixelA[RAD+3][RAD+1]; SortB[3] = pixelB[RAD+3][RAD+1];
			SortL[31] = pixelL[RAD-1][RAD-3]; SortA[4] = pixelA[RAD-1][RAD-3]; SortB[4] = pixelB[RAD-1][RAD-3];
			SortL[32] = pixelL[RAD  ][RAD-3]; SortA[5] = pixelA[RAD  ][RAD-3]; SortB[5] = pixelB[RAD  ][RAD-3];
			SortL[33] = pixelL[RAD+1][RAD-3]; SortA[6] = pixelA[RAD+1][RAD-3]; SortB[6] = pixelB[RAD+1][RAD-3];
			SortL[34] = pixelL[RAD-1][RAD+3]; SortA[7] = pixelA[RAD-1][RAD+3]; SortB[7] = pixelB[RAD-1][RAD+3];
			SortL[35] = pixelL[RAD  ][RAD+3]; SortA[8] = pixelA[RAD  ][RAD+3]; SortB[8] = pixelB[RAD  ][RAD+3];
			SortL[36] = pixelL[RAD+1][RAD+3]; SortA[0] = pixelA[RAD+1][RAD+3]; SortB[0] = pixelB[RAD+1][RAD+3];
			//This completes the 7-wide circular neighborhood - remainder of arrays are not used at this time
			SortArrays(SortL, SortA, SortB, arraysize); // sort on L, carry A, B along
			ResultLAB[x].L = SortL[medval];//result line gets the median value
			ResultLAB[x].A = SortA[medval];
			ResultLAB[x].B = SortB[medval];
		}//for x
		ConvertLineLABtoRGB(width, ResultLAB, ResultRGB);//convert back to RGB
		WriteResultLine(y, ResultRGB);// and write out line
	}//for y
	
mainexit:	//clean up everything we allocated
	for (i = 0; i < size; i++)
	{	if (RGBline[i]) DisposeAPointer(RGBline[i]);
		if (LABline[i]) DisposeAPointer(LABline[i]);
	}
	if (ResultRGB) DisposeAPointer(ResultRGB);
	if (ResultLAB) DisposeAPointer(ResultLAB);

	return result;											//pass any errors back
}//MainUserEntry

