
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 2.2.7.11 Rank Morphology */

#include "PhotoshopShell.h"

void Sort(float *array, long N); // Sort N elements in array into value order
void Sort(float *array, long N)
{	Boolean	done;
	long	jump, i, j;
	float	temp;

	jump = N;
	while (jump > 0)
	{	jump /= 2;
		do
		{	done = TRUE;
			for (j = 0; j < N-jump; j++)
			{	i = j + jump;
				if (array[j] > array[i])
				{	temp = array[j]; array[j] = array[i]; array[i] = temp;
					done = FALSE;
				}
			}
		} while (!done);
	}
}

typedef struct	// holds spherical LAB values for each pixel
{	float L, A, B;
}LABPixel;

typedef	RGBPixel *RGBLinePtr; // pointer for RGB pixel values
typedef LABPixel *LABLinePtr; // and for LAB values
typedef	float *FloatLinePtr; // and for a line of floats


void ConvertLineRGBtoLAB(long width, RGBPixel *RGBline, LABPixel *LABline);
void ConvertLineRGBtoLAB(long width, RGBPixel *RGBline, LABPixel *LABline)
{	
	long	x;
	for (x = 0; x < width; x++)
	{	// convert RGB to spherical LAB values
		LABline[x].L = (RGBline[x].red + RGBline[x].green + RGBline[x].blue)/3;
		LABline[x].A = (RGBline[x].red - RGBline[x].green);
		LABline[x].B = (RGBline[x].red + RGBline[x].green)/2 - RGBline[x].blue;
	}
}

void ConvertLineLABtoRGB(long width, LABPixel *LABline, RGBPixel *RGBline);
void ConvertLineLABtoRGB(long width, LABPixel *LABline, RGBPixel *RGBline)
{
	long	x;
	for (x = 0; x < width; x++)
	{	// convert from spherical LAB values to RGB
		RGBline[x].red =   LABline[x].L + LABline[x].A/2 + LABline[x].B/3;
		RGBline[x].green = LABline[x].L - LABline[x].A/2 + LABline[x].B/3;
		RGBline[x].blue =  LABline[x].L - 2 * LABline[x].B/3;
	}
}


///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	x, y, i, count;									//loop counter to traverse image and neighborhood
	float	SortArray[9];									//array to be sorted
	RGBPixel *line = NULL;
	FloatLinePtr tline[3];									//memory buffers for multiple lines
	FloatLinePtr rline;										//result line of floats
	LABPixel *rLAB;											//result line of LAB values
	Image_ID	Temp1, Temp2;								//temporary image pointers
	long	iterations = 3; // read this from a text file
	long	openclose = 2;  // and this: 0=erode, 1=dilate; 2=open, 3=close
	long	fref;

	if (OpenTextFileToRead("C:\\morph.txt", &fref) != noErr) // find file with control parameters
	{	result = fnfErr;
		goto mainexit;
	}
	else
	{	float	dummy; // to read from the text file
		result = ReadANumber(fref, &dummy); iterations = (long)dummy;// number of iterations
		result = ReadANumber(fref, &dummy); openclose = (long)dummy;//what operation to perform
		CloseTextFile(fref);
		if (result != noErr)
			goto mainexit;
	}

	GetOriginalDimensions(&width, &height);					//find out how big the image is

	result = MakeTemporaryImage(width, height, sizeof(float),"Temp1", &Temp1);
	result = MakeTemporaryImage(width, height, sizeof(float),"Temp2", &Temp2);
		//Allocate temporary image workspace so we can iterate
							
	line = CreateAPointer(width, sizeof(RGBPixel));			//for the original image
	for (i = 0; i < 3; i++)
		tline[i] = CreateAPointer(width, sizeof(float));	//for reading the temp image buffer
	rline = CreateAPointer(width, sizeof(float));			//and writing back
	rLAB = CreateAPointer(width, sizeof(LABPixel));			//for final rewrite

	// first pass - read in brightness. store in temp array
	for (y = 0; y < height; y++)
	{	if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadOriginalLine(y, line);
		for (x = 0; x < width; x++)
			rline[x] = (line[x].red+line[x].green+line[x].blue)/3;
		WriteTempImageLine(Temp1, y, rline);	
	}

	
	for (count = 0; count < iterations; count++) // repeat the selected number of times
	{	
		// first pass erosion or dilation
		for (y = 0; y < height; y++) // process the temp1 image one line at a time
		{	
			if (y == 0) ReadTempImageLine(Temp1, 0, tline[0]); // read three lines
			else ReadTempImageLine(Temp1, y-1, tline[0]);// boundary protection at the top
			if (y == height-1) ReadTempImageLine(Temp1, height-1, tline[2]); // and the bottom
			else ReadTempImageLine(Temp1, y+1, tline[2]);
			ReadTempImageLine(Temp1, y, tline[1]);
			
			for (x = 0; x < width; x++)// traverse each line a pixel at a time
			{	
				for (i = 0; i < 3; i++) // read neighborhood values into sorting array
				{	
					if (x == 0) SortArray[i] = tline[i][0];
					else SortArray[i] = tline[i][x-1]; // boundary protection at left
					if (x == width-1) SortArray[6+i] = tline[i][width-1];
					else SortArray[6+i] = tline[i][x+1];// and at right
					SortArray[3+i] = tline[i][x];
				}
				
				Sort (SortArray, 9); //sort the values into order
				
				// get the brightest or darkest value in the list and create the result line
				if ((openclose == 0) || (openclose == 2)) rline[x] = SortArray[0]; // erode (darkest)
				else rline[x] = SortArray[8]; // dilate (brightest)
			}
			WriteTempImageLine(Temp2, y, rline); // write the results back to the other temp image (temp2)
		}// for y
		
		for (y = 0; y < height; y++) // now copy back to Temp1 for next pass
		{	
			ReadTempImageLine(Temp2, y, rline);
			WriteTempImageLine(Temp1, y, rline);
		}
	}

	// second pass iterative dilation or erosion only for open and close (skip for erode or dilate)
	if ((openclose == 2) || (openclose == 3))
	{	
		for (count = 0; count < iterations; count++)
		{	
			for (y = 0; y < height; y++) // process is just like that above
			{	
				if (y == 0) ReadTempImageLine(Temp1, 0, tline[0]);
				else ReadTempImageLine(Temp1, y-1, tline[0]);
				if (y == height-1) ReadTempImageLine(Temp1, height-1, tline[2]);
				else ReadTempImageLine(Temp1, y+1, tline[2]);
				ReadTempImageLine(Temp1, y, tline[1]);
				for (x = 0; x < width; x++)
				{	for (i = 0; i < 3; i++)
					{	if (x == 0) SortArray[i] = tline[i][0]; else SortArray[i] = tline[i][x-1];
						if (x == width-1) SortArray[6+i] = tline[i][width-1]; else SortArray[6+i] = tline[i][x+1];
						SortArray[3+i] = tline[i][x];
					}
					Sort (SortArray, 9);
					if (openclose == 3) rline[x] = SortArray[0]; // close = dilate then erode
					else rline[x] = SortArray[8]; // open = erode then dilate
				}
				WriteTempImageLine(Temp2, y, rline);
			}// for y
			for (y = 0; y < height; y++) // now copy back to Temp1 for next pass
			{	ReadTempImageLine(Temp2, y, rline);
				WriteTempImageLine(Temp1, y, rline);
			}
		}
	}//for count

	// done - write it back
	for (y = 0; y < height; y++)
	{	
		if (TestAbortSetProgressBar((float)(height+y)/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		//must merge intensity values from processing with original pixel color values
		ReadOriginalLine(y, line);
		ConvertLineRGBtoLAB(width, line, rLAB);//pixel values in LAB
		ReadTempImageLine(Temp1, y, rline);	//floats from processing
		for (x = 0; x < width; x++)
			rLAB[x].L = rline[x]; // replace L values with processed ones
		ConvertLineLABtoRGB(width, rLAB, line);//convert back to RGB
		WriteResultLine(y, line);//and rewrite
	}

mainexit:													//clean up everything we allocated
	if (line)
		DisposeAPointer(line);
	if (rline)
		DisposeAPointer(rline);
	for (i = 0; i < 3; i++)
		if (tline[i]) DisposeAPointer(tline[i]);

	return result;											//pass any errors back
}//MainUserEntry

