
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 2.2.7.13 Add Noise */

#include <time.h>									//needed only to initialize random number generator

#include "PhotoshopShell.h"

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;							//accumulate any errors that happened
	long	height, width;							//pixel dimensions of image
	long	x, y;									//loop counters to traverse image
	double	randscal = RAND_MAX;					//max integer 0x7FFFFFFF;
	float	bright, temp, magnitude = 20;			//maximum amount to add
	Boolean	RGB=FALSE;								//add to individual channels vs. luminance
	RGBPixel *line = NULL;							//memory buffer

	GetOriginalDimensions(&width, &height);			//find out how big the image is
	line = CreateAPointer(width, sizeof(RGBPixel));	//allocate memory for one row
	srand((unsigned)time(NULL));					//initialize random number generator

	for (y = 0; y < height; y++)					//traverse image one line at a time
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadOriginalLine(y, line);				//read a line
		for (x = 0; x < width; x++)
		{	float	r,g,b;
			r = line[x].red; 
			g = line[x].green;
			b = line[x].blue;
			if (RGB) // alter individual channels
			{
				temp = (float)(rand()/randscal - 0.5);//-0.5...+0.5
				r = r + 2 * magnitude * temp;		//modify red
				temp = (float)(rand()/randscal - 0.5);
				g = g + 2 * magnitude * temp;		//green
				temp = (float)(rand()/randscal - 0.5);
				b = b + 2 * magnitude * temp;		//and blue independently
			}
			else // modify luminance without color shifts
			{
				temp = (float)(rand()/randscal - 0.5);//-0.5...+0.
				bright = (r + g + b)/3;				// or other weight
				temp = 2 * magnitude * temp;
				if (bright > 0)
				{	temp = (temp+bright) / bright;	//ratios preserve colors
					r *= temp;
					b *= temp;
					g *= temp;
				}
			}
			// protect against over/underflow
			if (r>255) r=255; if (r<0) r = 0;
			if (b>255) b=255; if (b<0) b = 0;
			if (g>255) g=255; if (g<0) g = 0;
			line[x].red = r;
			line[x].green = g;
			line[x].blue = b;			
		}
		WriteResultLine(y, line);				//write changed line back
	}//for y

mainexit:										//clean up everything we allocated
	if (line)
		DisposeAPointer(line);

	return result;								//pass any errors back
}//MainUserEntry

