
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 2.2.7.8 Hybrid Median */

#include "PhotoshopShell.h"

typedef struct // struct to hold spherical LAB pixel values
{	float L, A, B;
}LABPixel;

typedef	RGBPixel *RGBLinePtr; // pointer to hold a line of RGB pixel values
typedef LABPixel *LABLinePtr; // and a line of spherical LAB values


void ConvertLineRGBtoLAB(long width, RGBPixel *RGBline, LABPixel *LABline);
void ConvertLineRGBtoLAB(long width, RGBPixel *RGBline, LABPixel *LABline)
{
	long	x;
	for (x = 0; x < width; x++)
	{	// convert RGB to spherical LAB values
		LABline[x].L = (RGBline[x].red + RGBline[x].green + RGBline[x].blue)/3;
		LABline[x].A = (RGBline[x].red - RGBline[x].green);
		LABline[x].B = (RGBline[x].red + RGBline[x].green)/2 - RGBline[x].blue;
	}
}

void ConvertLineLABtoRGB(long width, LABPixel *LABline, RGBPixel *RGBline);
void ConvertLineLABtoRGB(long width, LABPixel *LABline, RGBPixel *RGBline)
{
	long	x;
	for (x = 0; x < width; x++)
	{	// convert from spherical LAB values to RGB
		RGBline[x].red =   LABline[x].L + LABline[x].A/2 + LABline[x].B/3;
		RGBline[x].green = LABline[x].L - LABline[x].A/2 + LABline[x].B/3;
		RGBline[x].blue =  LABline[x].L - 2 * LABline[x].B/3;
	}
}

//sorting routine sorts values in array1 (brightness values) but also rearranges the values
//in arrays 2 and 3 (color values) so they are carried along.
void SortArrays(float *array1, float *array2, float *array3, long N);
void SortArrays(float *array1, float *array2, float *array3, long N)
{	// general purpose sort can handle various size arrays
	Boolean	done;
	long	jump, i, j;
	float	temp;

	jump = N;
	while (jump > 0)
	{	jump /= 2;
		do
		{	done = TRUE;
			for (j = 0; j < N-jump; j++)
			{	i = j + jump;
				if (array1[j] > array1[i])
				{	//sort on array 1 but carry 2, 3 values along
					temp = array1[j]; array1[j] = array1[i]; array1[i] = temp;
					temp = array2[j]; array2[j] = array2[i]; array2[i] = temp;
					temp = array3[j]; array3[j] = array3[i]; array3[i] = temp;
					done = FALSE;
				}
			}
		} while (!done);
	}
}
#define RAD 2 // neighborhood size = 5x5 (central pixel plus 2 in all directions)
#define size 2*RAD+1
///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	x, y;											//loop counters to traverse image
	long	i, j, h, k;										//loop counters for neighborhood
	float	pixelL[size][size], pixelA[size][size], pixelB[size][size];		//pixel values
	float	SortL[9], SortA[9], SortB[9];					//sort + and x sub-nborhoods
	float	SrtL[3], SrtA[3], SrtB[3];						//final median sort
	RGBLinePtr RGBline[size];								//memory buffer - 5 lines of RGB
	LABLinePtr LABline[size];								//and spherical LAB pixels
	RGBPixel *ResultRGB;									//resulting pixel values in RGB
	LABPixel *ResultLAB;									//intermediate result in LAB

	GetOriginalDimensions(&width, &height);					//find out how big the image is

	for (i = 0; i < size; i++)								//initialize pointers to NULL
	{														//for easier cleanup below
		RGBline[i] = NULL;
		LABline[i] = NULL;
	}
	ResultRGB = NULL;
	ResultLAB = NULL;
	
	for (i = 0; i < size; i++)
	{
		RGBline[i] = CreateAPointer(width, sizeof(RGBPixel));
		LABline[i] = CreateAPointer(width, sizeof(LABPixel));	//Allocate memory for mulitple rows
	}
	ResultRGB = CreateAPointer(width, sizeof(RGBPixel));	//and for results
	ResultLAB = CreateAPointer(width, sizeof(LABPixel));

	for (y = 0; y < height; y++)//traverse image line by line
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{	//See if user canceled and update progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		for (i = 0; i < size; i++)// read multiple lines of pixels
		{
			h = y + i - RAD;//vertical offset
			if (h<0) ReadOriginalLine(0, RGBline[i]); // boundary protection at top
			else if (h>=height) ReadOriginalLine(height-1, RGBline[i]);// and bottom
			else ReadOriginalLine(h, RGBline[i]);
		}
		for (i = 0; i < size; i ++)//convert pixel values to LAB
			ConvertLineRGBtoLAB(width, RGBline[i], LABline[i]);
		
		for (x = 0; x < width; x++)//traverse each line from left to right
		{
			for (i = 0; i < size; i++)
				for (j = 0; j < size; j++)
				{
					k = x + j - RAD;//horizontal offset
					if (k<0) //boundary protection at left
					{	pixelL[i][j] = LABline[i][0].L;
						pixelA[i][j] = LABline[i][0].A;
						pixelB[i][j] = LABline[i][0].B;
					}
					else if (k>=width) //and right
					{	pixelL[i][j] = LABline[i][width-1].L;
						pixelA[i][j] = LABline[i][width-1].A;
						pixelB[i][j] = LABline[i][width-1].B;
					}
					else 
					{	pixelL[i][j] = LABline[i][k].L;
						pixelA[i][j] = LABline[i][k].A;
						pixelB[i][j] = LABline[i][k].B;
					}
				}
			//Copy pixel values into arrays - first the x sub-nborhood
			SortL[0] = pixelL[0][0]; SortA[0] = pixelA[0][0]; SortB[0] = pixelB[0][0];
			SortL[1] = pixelL[1][1]; SortA[1] = pixelA[1][1]; SortB[1] = pixelB[1][1];
			SortL[2] = pixelL[2][2]; SortA[2] = pixelA[2][2]; SortB[2] = pixelB[2][2];
			SortL[3] = pixelL[3][3]; SortA[3] = pixelA[3][3]; SortB[3] = pixelB[3][3];
			SortL[4] = pixelL[4][4]; SortA[4] = pixelA[4][4]; SortB[4] = pixelB[4][4];
			SortL[5] = pixelL[0][4]; SortA[5] = pixelA[0][4]; SortB[5] = pixelB[0][4];
			SortL[6] = pixelL[1][3]; SortA[6] = pixelA[1][3]; SortB[6] = pixelB[1][3];
			SortL[7] = pixelL[3][1]; SortA[7] = pixelA[3][1]; SortB[7] = pixelB[3][1];
			SortL[8] = pixelL[4][0]; SortA[8] = pixelA[4][0]; SortB[8] = pixelB[4][0];
			SortArrays(SortL, SortA, SortB, 9); // sort on L, carry A, B along
			SrtL[0] = SortL[4]; SrtA[0] = SortA[4]; SrtB[0] = SortB[4];
			//now + sub-nborhood (could also introduce 22 and 68 degree sub-nborhoods)
			SortL[0] = pixelL[0][2]; SortA[0] = pixelA[0][2]; SortB[0] = pixelB[0][2];
			SortL[1] = pixelL[1][2]; SortA[1] = pixelA[1][2]; SortB[1] = pixelB[1][2];
			SortL[2] = pixelL[2][2]; SortA[2] = pixelA[2][2]; SortB[2] = pixelB[2][2];
			SortL[3] = pixelL[3][2]; SortA[3] = pixelA[3][2]; SortB[3] = pixelB[3][2];
			SortL[4] = pixelL[4][2]; SortA[4] = pixelA[4][2]; SortB[4] = pixelB[4][2];
			SortL[5] = pixelL[2][4]; SortA[5] = pixelA[2][4]; SortB[5] = pixelB[2][4];
			SortL[6] = pixelL[2][3]; SortA[6] = pixelA[2][3]; SortB[6] = pixelB[2][3];
			SortL[7] = pixelL[2][1]; SortA[7] = pixelA[2][1]; SortB[7] = pixelB[2][1];
			SortL[8] = pixelL[2][0]; SortA[8] = pixelA[2][0]; SortB[8] = pixelB[2][0];
			SortArrays(SortL, SortA, SortB, 9);
			SrtL[1] = SortL[4]; SrtA[1] = SortA[4]; SrtB[1] = SortB[4];
			// and include original pixel
			SrtL[2] = pixelL[2][2]; SrtA[2] = pixelA[2][2]; SrtB[2] = pixelB[2][2];
			SortArrays(SrtL, SrtA, SrtB, 3);
			ResultLAB[x].L = SrtL[1];// resulting hybrid median pixel in LAB coordinates
			ResultLAB[x].A = SrtA[1];
			ResultLAB[x].B = SrtB[1];
		}//for x
		ConvertLineLABtoRGB(width, ResultLAB, ResultRGB);//convert to RGB
		WriteResultLine(y, ResultRGB);// and write back
	}//for y
	
mainexit:	//clean up everything we allocated
	for (i = 0; i < size; i++)
	{	if (RGBline[i]) DisposeAPointer(RGBline[i]);
		if (LABline[i]) DisposeAPointer(LABline[i]);
	}
	if (ResultRGB) DisposeAPointer(ResultRGB);
	if (ResultLAB) DisposeAPointer(ResultLAB);

	return result;											//pass any errors back
}//MainUserEntry

