
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 2.3.4.6 Local Equalization */

#include "PhotoshopShell.h"

typedef struct// to hold LAB pixel values
{	float L, A, B;
}LABPixel;

typedef	RGBPixel *RGBLinePtr;//pointer to a line of RGB pixels
typedef LABPixel *LABLinePtr;//pointer to a line of LAB pixels

void ConvertLineRGBtoLAB(long width, RGBPixel *RGBline, LABPixel *LABline);
void ConvertLineRGBtoLAB(long width, RGBPixel *RGBline, LABPixel *LABline)
{	long	x;
	for (x = 0; x < width; x++)
	{	// convert RGB to spherical LAB values
		LABline[x].L = (RGBline[x].red + RGBline[x].green + RGBline[x].blue)/3;
		LABline[x].A = (RGBline[x].red - RGBline[x].green);
		LABline[x].B = (RGBline[x].red + RGBline[x].green)/2 - RGBline[x].blue;
	}
}

void ConvertLineLABtoRGB(long width, LABPixel *LABline, RGBPixel *RGBline);
void ConvertLineLABtoRGB(long width, LABPixel *LABline, RGBPixel *RGBline)
{	long	x;
	for (x = 0; x < width; x++)
	{	// convert from spherical LAB values to RGB
		RGBline[x].red =   LABline[x].L + LABline[x].A/2 + LABline[x].B/3;
		RGBline[x].green = LABline[x].L - LABline[x].A/2 + LABline[x].B/3;
		RGBline[x].blue =  LABline[x].L - 2 * LABline[x].B/3;
	}
}
#define RAD 4 // neighborhood size = 9x9 nborhood (central pixel plus 4 on all sides)
#define size 2*RAD+1

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	x, y;											//loop counters to traverse image
	long	i, j, h, k;										//loop counters for neighborhood
	long	countdarker, countequal;						//counters for pixels in local histogram
	float	temp, scale;									//used for brightness
	float	pixel[size][size];								//pixel values
	RGBLinePtr RGBline[size];								//memory buffer - multiple lines
	LABLinePtr LABline[size];								//lines of pixels after LAB conversion
	RGBPixel *ResultRGB;									//final result
	LABPixel *ResultLAB;									//intermediate result

	GetOriginalDimensions(&width, &height);					//find out how big the image is

	scale = 3.0; // an arbitrary constant

	for (i = 0; i < size; i++)	//initialize pointers to null for easier cleanup below
	{	RGBline[i] = NULL;
		LABline[i] = NULL;
	}
	ResultRGB = NULL;
	ResultLAB = NULL;
	
	for (i = 0; i < size; i++)
	{	RGBline[i] = CreateAPointer(width, sizeof(RGBPixel));
		LABline[i] = CreateAPointer(width, sizeof(LABPixel));	//allocate memory for multiple rows
	}
	ResultRGB = CreateAPointer(width, sizeof(RGBPixel));	//and for results
	ResultLAB = CreateAPointer(width, sizeof(LABPixel));

	for (y = 0; y < height; y++) // first pass - scan line by line to get min & max
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{	//See if user canceled and update progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		for (i = 0; i < size; i++) // read multiple lines
		{
			h = y + i - RAD; //vertical offset
			if (h<0) ReadOriginalLine(0, RGBline[i]); // boundary protection at top
			else if (h>=height) ReadOriginalLine(height-1, RGBline[i]);//and bottom
			else ReadOriginalLine(h, RGBline[i]);
		}
		for (i = 0; i < size; i ++)
			ConvertLineRGBtoLAB(width, RGBline[i], LABline[i]);//work with LAB values
		
		for (x = 0; x < width; x++) //scan line pixel by pixel
		{
			for (i = 0; i < size; i++)
				for (j = 0; j < size; j++) // access neighborhood
				{
					k = x + j - RAD;//horizontal offset
					if (k<0) pixel[i][j] = LABline[i][0].L;//boundary protection at left
					else if (k>=width) pixel[i][j] = LABline[i][width-1].L;//and right
					else pixel[i][j] = LABline[i][k].L;
				}
			// count the number of pixels darker than and equal to current center pixel
			countequal = 0; countdarker = 0;
			temp = pixel[RAD][RAD];// current center
			for (i = 0; i < size; i++)
				for (j = 0; j < size; j++) // scan neighborhood
				{
					if (pixel[i][j] < temp) countdarker ++;
					else if (pixel[i][j] == temp) countequal ++;
				}
			temp = scale * (float)(countdarker + countequal/2); // new value
			// rank position of current pixel in its neighborhood
			ResultLAB[x].L = temp; // combine new brightness with original color info
			ResultLAB[x].A = LABline[1][x].A;
			ResultLAB[x].B = LABline[1][x].B;
		}//for x
		ConvertLineLABtoRGB(width, ResultLAB, ResultRGB);
		WriteResultLine(y, ResultRGB);//and write it back
	}//for y
	
mainexit:	//clean up everything we allocated
	for (i = 0; i < size; i++)
	{	if (RGBline[i]) DisposeAPointer(RGBline[i]);
		if (LABline[i]) DisposeAPointer(LABline[i]);
	}
	if (ResultRGB) DisposeAPointer(ResultRGB);
	if (ResultLAB) DisposeAPointer(ResultLAB);

	return result;											//pass any errors back
}//MainUserEntry

