
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 3.2.5.1 Wavelet Transform */

#include "PhotoshopShell.h"

void Daub_4 (float *data, long N, long direction);//Daubechies 4-wide Wavelet
void Daub_4 (float *data, long N, long direction)
{
	float h0,h1,h2,h3;//four weights
	long i,j,nover2;//loop indices
	float *tmp;//temporary sum array
	double rt3, rt2;//constants

	// define constants and the coefficients
 	rt3 = sqrt(3.0);
	h0 = (float)((1.0 + rt3)/4.0);         h1 = (float)((3.0 + rt3)/4.0);
	h2 = (float)((3.0 - rt3)/4.0);         h3 = (float)((1.0 - rt3)/4.0);
	rt2 = sqrt(2.0);
	h0 /= (float)rt2;  h1 /= (float)rt2;   h2 /= (float)rt2;  h3 /= (float)rt2;
	nover2 = N/2;
	
	tmp = CreateAPointer(N, sizeof(float));//allocate memory for results
	
	if (direction>=0) // forward transform
	{	i = 0;
	  	for (j = 0; j < N-3; j += 2) //traverse the line of data and built temp values
	  	{	
	  		tmp[i] = h0*data[j] + h1*data[j+1] + h2*data[j+2] + h3*data[j+3];
	    	tmp[i+nover2] = 
				-(h0*data[j+3] + h2*data[j+1]) + (h1*data[j+2] + h3*data[j]);
	   		i++;
		}
		// End around overlap
	    tmp[i] = h0*data[N-2] + h1*data[N-1] + h2*data[0] + h3*data[1];
	    tmp[i+nover2] =
	    	-(h0*data[1] + h2*data[N-1]) + (h1*data[0] + h3*data[N-2]);
	} else // reverse transform
	{	i = 0; j = 2;
	  	do
	  	{	
	  		tmp[j] = h2*data[i]+h1*data[i+nover2]+
		   		h0*data[(i+1)%nover2]+h3*data[(i+1)%nover2 + nover2];
	    	tmp[j+1] = h3*data[i]-h0*data[i+nover2]+
		   		h1*data[(i+1)%nover2]-h2*data[(i+1)%nover2 + nover2];
	    	j = (j+2)%N;
	    	i++;
	  	} while (i < nover2);
	}

	for (i=0; i<N; i++) data[i] = tmp[i];//pass back the results
	DisposeAPointer(tmp);
}

void Haar_2 (float *data, long N, long direction);//Haar 2-wide wavelet
void Haar_2 (float *data, long N, long direction)
{	// generate Haar coefficients for each iteration
	long i,j,nover2;//loop indices
	float h0,h1;//weights
	float *tmp;//pointer to temp results
	
	h0 = h1 = 0.5;//define constants
	nover2 = N/2;
	tmp = CreateAPointer(N, sizeof(float));//allocate memory
	
	if (direction >= 0)// forward
	{	i = 0;
		for (j = 0; j < N-1; j += 2)
		{	tmp[i]		 = h0 * data[j] + h1 * data[j+1];
			tmp[i+nover2]= h0 * data[j] - h1 * data[j+1];
			i++;
		}
	}
	else // direction==INVERSE
	{	i = j = 0;
		h0 = h1 = 1.0;
		do
		{	tmp[j]		= h0 * data[i] + h1 * data[i+nover2];
			tmp[j+1]	= h0 * data[i] - h1 * data[i+nover2];
			j += 2; i ++;
		}while (i < nover2);
	}
	for (i=0; i<N; i++)
	{
		data[i] = tmp[i];//copy results back
	}
	DisposeAPointer(tmp);
}

void fwt_1d (float *data, long N, long direction);//wavelet on a row or column of values
void fwt_1d (float *data, long N, long direction)
{
	long thisn,cptr;
	
	{	if (N>2)
		{
			if (direction >= 0)//forward
			{
				thisn = N;
				while (thisn>=2) // successive halving
				{	Haar_2(data,thisn,direction);
					thisn /= 2;
				}
			} else // direction == inverse
			{
				thisn = 2;
				cptr = 0;
				while (thisn<=N) // successive doubling
				{
					Haar_2(data,thisn,direction);
					thisn *= 2; cptr ++;
				}
			}
		}
	}
}

void fwt_2d (long wfref, long dimension, long direction);//wavelet on a 2D array
void fwt_2d (long wfref, long dimension, long direction)
{	long i,j;
	float *temp;
	// transform the rows, then columns
	temp = CreateAPointer(dimension, sizeof(float));
	for (i = 0; i < dimension; i++)
	{
		ReadTempImageLine(wfref, i, temp);//read a row
		fwt_1d(temp, dimension, direction);//transform it
		WriteTempImageLine(wfref, i, temp);//write it back
	}
	for (j = 0; j < dimension; j++)
	{
		ReadTempImageColumn(wfref, j, temp);//read a column
		fwt_1d(temp, dimension, direction);//transform it
		WriteTempImageColumn(wfref, j, temp);//write it back
	}
	DisposeAPointer(temp);
}

#define FORWARD 1
#define REVERSE -1

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;								//accumulate any errors that happened
	long	height, width;								//pixel dimensions of image
	long	x, y;										//loop counter to traverse image
	RGBPixel *line = NULL;								//line of pixels
	float *rline = NULL;								//line of brightness values only
	Image_ID	WVT;									//temporary image
	float val, scale, min, max;							//perform autoscaling

	GetOriginalDimensions(&width, &height);					//find out how big the image is

	if ((width != 256) || (height != 256))
	{
		ErrorMessage("This routine is written for just 256x256 images only.");
		result = userCanceledErr;
		return result;
	}
	result = MakeTemporaryImage(width, height, sizeof(float),"Wave", &WVT);
		//Allocate temporary image workspace
							
	line = CreateAPointer(width, sizeof(RGBPixel));			//for the original image
	rline = CreateAPointer(width, sizeof(float));			//for the transform

	//read in brightness. store in temp array
	for (y = 0; y < height; y++)
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadOriginalLine(y, line);
		for (x = 0; x < width; x++)
			rline[x] = (line[x].red+line[x].green+line[x].blue)/3;//get brightness values

		WriteTempImageLine(WVT, y, rline);	//copy into a temp image
	}

	fwt_2d (WVT, width, FORWARD); //transform it


	// write back
	{	min = (float)1e20, max = (float)-1e20; // must autoscale the array
		for (y = 0; y < height; y++)
		{
			ReadTempImageLine(WVT, y, rline);
			for (x = 0; x < width; x++)
			{
				val = rline[x];
				if (val>max) max = val;//find min and max
				if (val<min) min = val;
			}
		}
		if (max>min)
		{
			scale = (float)255.0 / (max - min);
			for (y = 0; y < height; y++)
			{
				if (TestAbortSetProgressBar((float)(height+y)/(float)(height*2)))
				{// test for cancel and advance progress bar
					result = userCanceledErr;
					goto mainexit;
				}
				ReadTempImageLine(WVT, y, rline);	
				for (x = 0; x < width; x++)
				{	//rescale and rewrite results as grey scale image
					val = rline[x];
					val = scale * (val - min);
					line[x].red = line[x].green = line[x].blue = val;
				}
				WriteResultLine(y, line);
			} // for y
		} // if max>min
	} // write back

mainexit:		//clean up everything we allocated
	if (line) DisposeAPointer(line);
	if (rline) DisposeAPointer(rline);

	return result;		//pass any errors back
}//MainUserEntry

