
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 4.1.5.2 Threshold */

#include "PhotoshopShell.h"

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	x, y, bright, i;								//loop counters for image, indices for histogram
	long	t, thr1, thr2, thr3, thr4, test1;				//various threshold values
	long	Histogram[256];									//fixed number of bins
	float	BSum1, BSum2, BSum3, WSum1, WSum2, WSum3;		//sums used for various statistics
	float	BMean, WMean, BEntr, WEntr, BCumul, WCumul;
	float	Total, threshold, test2, test3, test4;
	float	value, ratio, Denom, Numer, BVar, WVar, FSum;
	long	method = 1;	// which of the 4 auto threshold methods to use (or read from a text file)
	RGBPixel *line = NULL;									//memory buffer

	GetOriginalDimensions(&width, &height);					//find out how big the image is
	Total = (float)(height * width);

	line = CreateAPointer(width, sizeof(RGBPixel));			//Allocate memory for one row

	for (bright = 0; bright < 256; bright++)
		Histogram[bright] = 0;								//clear the histogram array			

	for (y = 0; y < height; y++)
	{	//scan image line by line, pixel by pixel, to build histogram
		if (TestAbortSetProgressBar((float)y/(float)height))
		{	//test cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadOriginalLine(y, line);//read a line
		for (x = 0; x < width; x++)
		{	// average brightness, could also use other weights
			bright = (long)((line[x].red + line[x].green + line[x].blue)/3);
			Histogram[bright]++;//Increment histogram bin count
		}//for x
	}//for y
	
	// initialize comparison test values for all four methods
	thr1 = 0; test1 = 0; // midpoint method (1)
	thr2 = 0; test2 = 0; // t-statistic method (2)
	thr3 = 0; test3 = 0; // entropy method (3)
	thr4 = 0; test4 = (float)1e20; // fuzzy probability method (4)
	for (t = 1; t < 255; t++) // test every possible threshold setting
	{	WSum1 = WSum2 = WSum3 = BSum1 = BSum2 = BSum3 = 0.0;
		WEntr = BEntr = 0.0;
		BCumul = 0.0;
		for (i = 0; i <= t; i++)
			BCumul += (float)Histogram[i]; // count darker pixels
		WCumul = Total - BCumul; // and brighter ones
		for (i = 0; i <= t; i++) // sum background (darker)
		{	BSum1 += (float)Histogram[i];
			BSum2 += (float)(i * Histogram[i]);
			BSum3 += (float)(i * i * Histogram[i]);
			if ((Histogram[i] > 0) && (BCumul > 0))
			{	ratio = (float)Histogram[i]/BCumul;
				BEntr = BEntr - ratio * (float)log(ratio);//entropy
			}
		}
		for (i = t+1; i <= 255; i++) // sum features or bright
		{	WSum1 += (float)Histogram[i];
			WSum2 += (float)(i * Histogram[i]);
			WSum3 += (float)(i * i * Histogram[i]);
			if ((Histogram[i] > 0) && (WCumul > 0))
			{	ratio = (float)Histogram[i]/WCumul;
				WEntr = WEntr - ratio * (float)log(ratio);//entropy
			}
		}
		if ((BSum1>1000) && (WSum1>1000)) //skip empty parts of histogram
		{	BMean = BSum2/BSum1;//mean values
			WMean = WSum2/WSum1;
			test1 = (long)((BMean + WMean) / 2);
			if (test1 == t) thr1 = t; //midpoint method (1)
			BVar = BSum3/BSum1 - BMean*BMean; // variances
			WVar = WSum3/WSum1 - WMean*WMean;
			Numer = (WMean - BMean); Numer = Numer * Numer;
			Denom = (WVar/WSum1) + (BVar/BSum1);
			ratio = Numer/Denom; // actually the square of the t-statistic
			if (ratio > test2)
			{	test2 = ratio;
				thr2 = t;	//t-statistic method (2)
			}
			if ((BEntr + WEntr) > test3)
			{	test3 = (BEntr + WEntr);
				thr3 = t;	//max entropy method (3)
			}
			FSum = 0;
			for (i = 0; i < 256; i++) // sum fuzzy probability of set membership
			{	float	fuzz;
				if (i <= t) fuzz = (float)1.0 - (float)(fabs((float)i - BMean)/256);
				else		fuzz = (float)1.0 - (float)(fabs((float)i - WMean)/256);
				FSum -= fuzz * (float)log(fuzz) * (float)Histogram[i]/Total;
			}
			if (FSum < test4)
			{	test4 = FSum;
				thr4 = t;	//minimum fuzziness method (4)
			}
		}
	}// for t
	switch(method) // all four have been calculated, select one to apply
	{	case 1:	threshold = (float)thr1;
			break;
		case 2:	threshold = (float)thr2;
			break;
		case 3:	threshold = (float)thr3;
			break;
		case 4:	threshold = (float)thr4;
			break;
	}
	//apply threshold and write binary image
	for (y = 0; y < height; y++)
	{	//scan image line by line
		if (TestAbortSetProgressBar((float)y/(float)height))
		{	//test cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadOriginalLine(y, line);//read a line
		for (x = 0; x < width; x++) // scan line pixel by pixel
		{	//calculate mean brightness
			value = (line[x].red + line[x].green + line[x].blue)/3;
			if (value <= threshold) value = 0.0; // apply threshold
			else value = 255.0;
			line[x].red = line[x].green = line[x].blue = value;//write black or white
		}//for x
		WriteResultLine(y, line);
	}//for y

mainexit:		//clean up everything we allocated
	if (line)
		DisposeAPointer(line);

	return result;	//pass any errors back
}//MainUserEntry

