
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 4.2.3.3 Binary Morphology */

#include "PhotoshopShell.h"

typedef	long *BLinePtr; // could do this with short or even char values

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;				//accumulate any errors that happened
	long	height, width;				//pixel dimensions of image
	long	x, y, i, count;				//loop counter to traverse image
	long	NborArray[8];				//array of eight touching neighbors
	float	brite;						//mean brightness value
	long	total;
	RGBPixel *line = NULL;				//memory buffer for RGB pixels
	BLinePtr tline[3];					//memory buffers for black/white values
	BLinePtr rline;						//result line of black/white values
	Image_ID	Temp1, Temp2;			//needs two temp images
	long	iterations = 3; // read this from a text file
	long	openclose = 2; 	// and this: 0=erode, 1=dilate; 2=open, 3=close
	long	fref;			//file pointer for input

	if (OpenTextFileToRead("C:\\morph.txt", &fref) != noErr)// read the control values
	{	result = fnfErr;
		goto mainexit;
	}
	else
	{
		float	dummy; // to read from the text file
		result = ReadANumber(fref, &dummy); iterations = (long)dummy;
		result = ReadANumber(fref, &dummy); openclose = (long)dummy;
		CloseTextFile(fref);
		if (result != noErr)
			goto mainexit;
	}

	GetOriginalDimensions(&width, &height);				//find out how big the image is

	result = MakeTemporaryImage(width, height, sizeof(long),"Temp1", &Temp1);
	result = MakeTemporaryImage(width, height, sizeof(long),"Temp2", &Temp2);
		//Allocate temporary image workspaces
							
	line = CreateAPointer(width, sizeof(RGBPixel));		//for the original image (RGB pixels)
	for (i = 0; i < 3; i++)
		tline[i] = CreateAPointer(width, sizeof(long));	//for reading the temp image buffer
	rline = CreateAPointer(width, sizeof(long));		//and writing back

	// first pass - read in brightness. store in temp array
	for (y = 0; y < height; y++)
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadOriginalLine(y, line);
		for (x = 0; x < width; x++)
		{	brite = (line[x].red+line[x].green+line[x].blue)/3;
			if (brite > 0.5) rline[x] = 1; else rline[x] = 0; // binary values
		}
		WriteTempImageLine(Temp1, y, rline);	//save in first temp image array
	}

	// iterative dilation or erosion
	for (count = 0; count < iterations; count++)//number of iterations
	{
		for (y = 0; y < height; y++)
		{	//read three lines for neighborhood
			if (y == 0) ReadTempImageLine(Temp1, 0, tline[0]); //boundary protection at top
			else ReadTempImageLine(Temp1, y-1, tline[0]);
			if (y == height-1) ReadTempImageLine(Temp1, height-1, tline[2]);//and bottom
			else ReadTempImageLine(Temp1, y+1, tline[2]);
			ReadTempImageLine(Temp1, y, tline[1]);
			for (x = 0; x < width; x++)
			{	//for each pixel on the line, get the 8 touching neighbors
				for (i = 0; i < 3; i++)
				{
					if (x == 0) NborArray[i] = tline[i][0];
					else NborArray[i] = tline[i][x-1];
					if (x == width-1) NborArray[3+i] = tline[i][width-1];
					else NborArray[3+i] = tline[i][x+1];
				}				
				NborArray[6] = tline[0][x]; NborArray[7] = tline[2][x];
				total = 0;// count the number of neighbors with opposite color
				for (i = 0; i < 8; i++)
					total += NborArray[i];
				if ((tline[1][x] == 1) && ((openclose == 0) || (openclose == 2))) // erode step
				{
					if (total != 8) rline[x] = 0;
					else rline[x] = 1;
				}
				else if ((tline[1][x] == 0) && ((openclose == 1) || (openclose == 3))) // dilate step
				{	
					if (total > 0) rline[x] = 1;
					else rline[x] = 0;
				}
			}
			WriteTempImageLine(Temp2, y, rline); //write result to second temp image array
		}// for y

		for (y = 0; y < height; y++) // now copy back to Temp1 for next pass
		{	
			ReadTempImageLine(Temp2, y, rline);
			WriteTempImageLine(Temp1, y, rline);
		}
	}

	// second pass iterative dilation or erosion only for open and close
	if ((openclose == 2) || (openclose == 3))
	{	//identical code to that above
		for (count = 0; count < iterations; count++)
		{
			for (y = 0; y < height; y++)
			{
				if (y == 0) ReadTempImageLine(Temp1, 0, tline[0]);
				else ReadTempImageLine(Temp1, y-1, tline[0]);
				if (y == height-1) ReadTempImageLine(Temp1, height-1, tline[2]);
				else ReadTempImageLine(Temp1, y+1, tline[2]);
				ReadTempImageLine(Temp1, y, tline[1]);
				for (x = 0; x < width; x++)
				{
					for (i = 0; i < 3; i++)
					{
						if (x == 0) NborArray[i] = tline[i][0];
						else NborArray[i] = tline[i][x-1];
						if (x == width-1) NborArray[3+i] = tline[i][width-1];
						else NborArray[3+i] = tline[i][x+1];
					}				
					NborArray[6] = tline[0][x]; NborArray[7] = tline[2][x];
					total = 0;
					for (i = 0; i < 8; i++)// count neighbors
						total += NborArray[i];
					if ((tline[1][x] == 1) && (openclose == 3)) // erode step of closing
					{
						if (total != 8) rline[x] = 0;
						else rline[x] = 1;
					}
					else if ((tline[1][x] == 0) && (openclose == 2)) // dilate step of opening
					{
						if (total > 0) rline[x] = 1;
						else rline[x] = 0;
					}
				}
				WriteTempImageLine(Temp2, y, rline);
			}// for y
			
			for (y = 0; y < height; y++)
			{
				ReadTempImageLine(Temp2, y, rline);
				WriteTempImageLine(Temp1, y, rline);
			}
		}
	}

	// finally, write it back
	for (y = 0; y < height; y++)
	{
		RGBPixel	WhitePix, BlackPix;
		WhitePix.red = WhitePix.green = WhitePix.blue = 255;
		BlackPix.red = BlackPix.green = BlackPix.blue = 0;

		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadTempImageLine(Temp1, y, rline);
		for (x = 0; x < width; x++)
		{
			if (rline[x] == 0) line[x] = BlackPix;
			else line[x] = WhitePix;
		}
		WriteResultLine(y, line);
	}

mainexit:	//clean up everything we allocated
	if (line)
		DisposeAPointer(line);
	if (rline)
		DisposeAPointer(rline);
	for (i = 0; i < 3; i++)
		if (tline[i]) DisposeAPointer(tline[i]);

	return result;	//pass any errors back
}//MainUserEntry

