
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 4.2.3.5 Euclidean Distance Map */

#include "PhotoshopShell.h"

typedef  float *FloatPtr; // for EDM values

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	x, y;											//loop counters to traverse image
	float	val, max, brite;								//scale final brightness values
	float	nbr1, nbr2, nbr3, nbr4;							//four neighbors above or below
	FloatPtr ThisLine, PrevLine;							//current line and line above or below
	Image_ID	Temp;										//temporary image to build EDM
	RGBPixel *ImageLine;									//pointer to RGB pixels

	GetOriginalDimensions(&width, &height);					//find out how big the image is

	ThisLine = CreateAPointer(width+2, sizeof(float));		//pad temp image by one pixel on all sides
	PrevLine = CreateAPointer(width+2, sizeof(float));
	ImageLine = CreateAPointer(width, sizeof(RGBPixel));

	result = MakeTemporaryImage(width+2, height+2, sizeof(float),"EDM", &Temp);
	
	for (x = 0; x < width+2; x++)
		ThisLine[x] = (float)1e20; // or 0.0 depending on boundary condition...
	WriteTempImageLine(Temp, 0, ThisLine); // initialize array
	WriteTempImageLine(Temp, height+1, ThisLine);

	for (y = 0; y < height; y++) // read original (binary) image, initialize temp image for EDM
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{	//See if user canceled and update progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		
		ReadOriginalLine(y, ImageLine);
		for (x = 0; x < width; x++)
		{	// assume if pixel is not black, it is treated as white
			brite = (ImageLine[x].red + ImageLine[x].green + ImageLine[x].blue)/3;
			if (brite < 0.5) ThisLine[x+1] = 0.0;
			else ThisLine[x+1] = 1.0;
		}		
		WriteTempImageLine(Temp, y+1, ThisLine);// temp image is 1 pixel larger on all sides than original
	}
	
	// downward sweep through temp image
	for (y = 1; y < height+1; y++)
	{
		ReadTempImageLine(Temp, y-1, PrevLine);//line above
		ReadTempImageLine(Temp, y,   ThisLine);//current line
		for (x = 1; x < width+1; x++)
		{	//set pixel to 1 or root2 larger than smallest of 4 left/above neighbors
			if (ThisLine[x] > 0)
			{
				nbr1 = PrevLine[x-1] + (float)1.414; val = nbr1;
				nbr2 = PrevLine[x  ] + (float)1.0;   if (val > nbr2) val = nbr2;
				nbr3 = PrevLine[x+1] + (float)1.414; if (val > nbr3) val = nbr3;
				nbr4 = ThisLine[x-1] + (float)1.0;   if (val > nbr4) val = nbr4;
				ThisLine[x] = val;
			}
			else ThisLine[x] = 0;	
		}
		WriteTempImageLine(Temp, y, ThisLine);//write it back
	}// for y

	// upward sweep - identical but comparison is to 4 right/lower neighbors
	for (y = height; y > 0; y--) // count up from bottom
	{
		ReadTempImageLine(Temp, y+1, PrevLine);//line below
		ReadTempImageLine(Temp, y,   ThisLine);//current line
		for (x = width; x > 0; x--) // and left from right edge
		{
			val = ThisLine[x];
			if (val > 0)
			{	//set pixel to 1 or root2 larger than smallest of 4 right/lower neighbors
				nbr1 = PrevLine[x+1] + (float)1.414; if (val > nbr1) val = nbr1;
				nbr2 = PrevLine[x  ] + (float)1.0;   if (val > nbr2) val = nbr2;
				nbr3 = PrevLine[x-1] + (float)1.414; if (val > nbr3) val = nbr3;
				nbr4 = ThisLine[x+1] + (float)1.0;   if (val > nbr4) val = nbr4;
				ThisLine[x] = val;
			}
			else ThisLine[x] = 0;
		}
		WriteTempImageLine(Temp, y, ThisLine);//write it back
	}// for y

	// finally autoscale and rewrite to host
	max = 1.0;
	for (y = 1; y < height+1; y++)
	{
		ReadTempImageLine(Temp, y, ThisLine);
		for (x = 1; x < width+1; x++)
			if (ThisLine[x] > max) max = ThisLine[x];//find max value
	}
	for (y = 0; y < height; y++)
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{	//See if user canceled and update progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadTempImageLine(Temp, y+1, ThisLine);
		for (x = 0; x < width; x++)
		{	brite = (float)255.0 * ThisLine[x+1] / max;//rescale to max value
			ImageLine[x].red = ImageLine[x].green = ImageLine[x].blue = brite;//write EDM as gray scale
		}// for x
		WriteResultLine(y, ImageLine);
	}//for y

mainexit:	//clean up everything we allocated
	DisposeAPointer(ThisLine);
	DisposeAPointer(PrevLine);
	DisposeAPointer(ImageLine);

	return result;	//pass any errors back
}//MainUserEntry

