
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 4.3.4.1 Watershed */

#include "PhotoshopShell.h"

typedef long  *ILinePtr; // pointer to EDM and binary values

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	x, y, i, j, k, level;							//loop counters
	long	nbr1,nbr2,nbr3,nbr4;							//neighbors for EDM construction
	long	index, count;
	long	feat = 0;
	float	brite;
	long	val, max = 0, min = 10000;
	Boolean found;											//flag
	RGBPixel *line = NULL;									//pointer to line of RGB pixels
	ILinePtr rline, tline;									//pointers to lines from temp images
	ILinePtr iline[7];
	Image_ID	EDM = 0, Bin1 = 0, Bin2 = 0;				//three temporary images (one EDM, two binaries)

	GetOriginalDimensions(&width, &height);					//find out how big the image is

	result = MakeTemporaryImage(width+2, height+2, sizeof(long), "EDM", &EDM);
	result = MakeTemporaryImage(width+2, height+2, sizeof(long), "Binary", &Bin1);
	result = MakeTemporaryImage(width+2, height+2, sizeof(long), "Dilate", &Bin2);
		//Allocate temporary image workspace
							
	line = CreateAPointer(width, sizeof(RGBPixel));			//for the original image
	for (i = 0; i < 7; i++)
		iline[i] = CreateAPointer(width+2, sizeof(long));	//for reading temp 
	rline = CreateAPointer(width+2, sizeof(long));
	tline = CreateAPointer(width+2, sizeof(long));			//and writing back
	
	for (x = 0; x < width+2; x++) // initialize line arrays
	{	rline[x] = 100000;
		tline[x] = 0;
		for (i = 0; i < 7; i++)
			iline[i][x] = 0;
	}
	WriteTempImageLine(EDM, 0, rline);	// and edges of EDM array
	WriteTempImageLine(EDM, height+1, rline);
	for (y = 0; y < height+2; y++)
	{
		WriteTempImageLine(Bin1, y, tline);
		WriteTempImageLine(Bin2, y, tline);
	}
		
	// read in image and initialize EDM array
	for (y = 0; y < height; y++)
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{	//See if user canceled and update progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadOriginalLine(y, line);
		for (x = 0; x < width; x++)
		{	brite = (line[x].red + line[x].green + line[x].blue)/3;
			if (brite < 0.5) rline[x+1] = 0; else rline[x+1] = 1;
		}		
		WriteTempImageLine(EDM, y+1, rline);
	}
	
	// build EDM - downward sweep
	for (y = 1; y < height+1; y++)
	{
		ReadTempImageLine(EDM, y-1, iline[0]);// read current line and line above
		ReadTempImageLine(EDM, y,   iline[1]);
		for (x = 1; x < width+1; x++)
		{
			if (iline[1][x] > 0)// set pixel 1 or 1.5 greater than smallest upper/left neighbor
			{
				nbr1 = iline[0][x-1] + 3; val = nbr1; // 3/2 = approximation to sqrt(2)
				nbr2 = iline[0][x  ] + 2; if (val > nbr2) val = nbr2;
				nbr3 = iline[0][x+1] + 3; if (val > nbr3) val = nbr3;
				nbr4 = iline[1][x-1] + 2; if (val > nbr4) val = nbr4;
				iline[1][x] = val;
			}
			else iline[1][x] = 0;	
		}
		WriteTempImageLine(EDM, y, iline[1]);
	}// for y

	// build EDM - upward sweep
	for (y = height; y > 0; y--)
	{
		ReadTempImageLine(EDM, y+1, iline[2]);//read current line and line below
		ReadTempImageLine(EDM, y,   iline[1]);
		for (x = width; x > 0; x--)
		{
			val = iline[1][x];
			if (val > 0)//set pixel to 1 or 1.5 greater than smallest right/lower neighbor
			{	nbr1 = iline[2][x+1] + 3; if (val > nbr1) val = nbr1;
				nbr2 = iline[2][x  ] + 2; if (val > nbr2) val = nbr2;
				nbr3 = iline[2][x-1] + 3; if (val > nbr3) val = nbr3;
				nbr4 = iline[1][x+1] + 2; if (val > nbr4) val = nbr4;
				iline[1][x] = val;
			}
			else iline[1][x] = 0;
		}
		WriteTempImageLine(EDM, y, iline[1]);
	}// for y

	// smooth EDM (optional step as described in the text)
	for (k = 0; k < 3; k++) // three iterations
	{
		for (y = 1; y < height; y++)
		{
			ReadTempImageLine(EDM, y-1, iline[0]);// average 3x3 neighborhood
			ReadTempImageLine(EDM, y  , iline[1]);
			ReadTempImageLine(EDM, y+1, iline[2]);
			for (x = 1; x < width; x++)
			{
				float	sum = 0;
				tline[x] = iline[1][x];
				for (i = 0; i < 3; i++)
					for (j = 0; j < 3; j++)
						sum += (float)iline[j][x-1+i];
					
				if (tline[x] > 0)
				{	sum /=9;
					tline[x] = (long)sum;
				}
			}// for x
			WriteTempImageLine(Bin2, y, tline);//write in temporary location
		}//for y
		
		for (y = 1; y < height; y++)
		{
			ReadTempImageLine(Bin2, y, tline); //copy back to EDM array
			WriteTempImageLine(EDM, y, tline);
		}
	}

	// scan EDM for UEPs and find max and min UEP values
	// size of search region determines spacing between UEPs
	{
		for (y = 3; y < height-2; y++)
		{
			for (j = 0; j < 7; j++)
				ReadTempImageLine(EDM, y-3+j, iline[j]);
			for (x = 3; x < width-2; x++)
			{
				val = iline[3][x]; // current pixel EDM value
				if (val > 0) // ignore background points
				{
					found = TRUE;
					for (i = 0; i < 7; i++)//compare to EDM value in 7x7 neighborhood
						for (j = 0; j < 7; j++)
						{	if ((i*j != 1) && (val < iline[j][x-3+i]))
							{	found = FALSE;
							}
						}
					if (found)//local maximum
					{
						if (val > max) max = val;//lowest, highest EDM peaks
						if (val < min) min = val;
						// mark this as a UEP in binary image 1
						ReadTempImageLine(Bin1, y, rline);
						rline[x] = 1;
						WriteTempImageLine(Bin1, y, rline);
					}
				}
			}
		}
	}

	// classic dilation of UEP's in Bin1 several times (optional step as described in text)
	// only to pixels with nonzero EDM values
	// the amount of dilation is determined by the minimum UEP value
	for (k = 0; k < min/3; k++)//number of iterations
	{
		for (y = 1; y < height; y++)
		{
			ReadTempImageLine(Bin1, y-1, iline[0]);//read three lines from binary
			ReadTempImageLine(Bin1, y  , iline[1]);
			ReadTempImageLine(Bin1, y+1, iline[2]);
			ReadTempImageLine(EDM, y, rline);//plus current line from EDM
			for (x = 1; x < width; x++)
			{
				tline[x] = iline[1][x];
				if ((rline[x] > 0) && (tline[x] == 0))
				{	count = 0;
					for (i = 0; i < 3; i++)
						for (j = 0; j < 3; j++)
						{	if (iline[j][x-1+i] == 1) count ++;
						}
					if (count > 0) tline[x] = 1;//classic dilation if any neighbors are found
				}
			}// for x
			WriteTempImageLine(Bin2, y, tline);// write to image 2
		}//for y
		for (y = 1; y < height; y++)
		{	ReadTempImageLine(Bin2, y, tline); // copy back from image 2 to image 1 for next iteration
			WriteTempImageLine(Bin1, y, tline);
		}
	}

	// feature numbering
	{	
		long featID = 0, newval, oldval;
		for (y = 1; y < height+1; y++)
		{
			ReadTempImageLine(Bin1, y-1, iline[0]);//read current line and line above
			ReadTempImageLine(Bin1, y  , iline[1]);
			for (x = 1; x < width+1; x++)
			{
				index = 0;//index depends on pattern of upper/left neighbors that are white
				val = iline[1][x];
				nbr1 = iline[1][x-1]; if (nbr1 != 0) index += 8;
				nbr2 = iline[0][x-1]; if (nbr2 != 0) index += 4;
				nbr3 = iline[0][x  ]; if (nbr3 != 0) index += 2;
				nbr4 = iline[0][x+1]; if (nbr4 != 0) index += 1;
				if (val != 0) // feature counting
				{	if (index == 0) // new feature - count and assign ID
					{	///increment feature number and assign
						featID ++;
						newval = featID;
					}
					// propagate id from previous pixel
					else if ((index == 1) || (index == 3) || (index == 7) || (index == 11) || (index == 15))
						newval = nbr4;
					else if ((index == 2) || (index == 6) || (index == 10) || (index == 14))
						newval = nbr3;
					else if ((index == 4) || (index == 12)) 
						newval = nbr2;
					else if (index == 8)
						newval = nbr1;
					else // test for merging id numbers
					{	newval = nbr4;
						if ((index == 9) || (index == 13)) oldval = nbr1;
						else oldval = nbr3;
						if (oldval != newval)
						{	// change id in prior lines
							//number --; // decrement feature count
							for (i = 1; i <= y; i ++)
							{	ReadTempImageLine(Bin1, i, rline);
								for (j = 1; j < width+1; j++)
									if (rline[j] == oldval) rline[j] = newval;
								WriteTempImageLine(Bin1, i, rline);
							}
						}
						for (j = 1; j < x; j++) // also change current line
							if (iline[1][j] == oldval) iline[1][j] = newval;
						ReadTempImageLine(Bin1, y-1, iline[0]); // reload changed line
					}
					iline[1][x] = newval;//assign feature id to pixel
				}//if not bg
			}// for x
			WriteTempImageLine(Bin1, y, iline[1]);//and write back to binary image 1
		}
	}

	// dilate only to pixels with nonzero EDM values
	// and only to pixels with no different neighboring IDs
	{
		long	Bin[9];
		for (level = max-1; level>0; level--)//work down from highest EDM pixel values
		{
			for (count = 0; count < 5; count++) // follow irregular shapes
			{	//changes = FALSE;
				for (y = 1; y < height+1; y++)
				{	ReadTempImageLine(Bin1, y-1, iline[0]);//read neighborhood from binary image 1
					ReadTempImageLine(Bin1, y  , iline[1]);
					ReadTempImageLine(Bin1, y+1, iline[2]);
					ReadTempImageLine(EDM, y, rline);//and read EDM value
					for (x = 1; x < width+1; x++)
					{	//collect pixel neighbors into Bin[] array
						if ((iline[1][x] == 0) && (rline[x]>= level))
						{
							for (i = 0; i < 3; i++)
								for (j = 0; j < 3; j++)
									Bin[i + 3*j] = iline[j][x-1+i];
							feat = 0; found = FALSE;
							for (i = 0; i < 9; i++)// test neighbors
							{	//see if any have a different ID number assigned
								if (Bin[i] != 0)
								{	if (feat==0) feat=Bin[i];
									else if (feat!=Bin[i]) found = TRUE;
								}
							}
							if (!found) // not adjacent to a different ID
							{
								iline[1][x] = feat;
								//changes = TRUE;
							}
						}
					}
					WriteTempImageLine(Bin1, y, iline[1]);
				}// for y
			}//while (changes);
		}
	}

	// last pass - write it back
	for (y = 0; y < height; y++)
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadTempImageLine(Bin1, y+1, rline);//read labelled binary image
		for (x = 0; x < width; x++)
		{	//if pixel is nonzero, set it to white
			if (rline[x+1] > 0)
				line[x].red = line[x].green = line[x].blue = 255;
			else
				line[x].red = line[x].green = line[x].blue = 0;
		}
		WriteResultLine(y, line);
	}


mainexit:						//clean up everything we allocated
	DisposeAPointer(line);
	DisposeAPointer(rline);
	DisposeAPointer(tline);
	for (i = 0; i < 7; i++)
		DisposeAPointer(iline[i]);

	return result;				//pass any errors back
}//MainUserEntry

