
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 4.3.4.2 Skeletonize */

#include "PhotoshopShell.h"

typedef	long *BLinePtr; // could do this with short or even char values

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	x, y, i, index;									//loop counter to traverse image
	float	brite;											//to read RGP pixels
	Boolean	changes;
	RGBPixel *line = NULL;									//line of RGB pixels
	BLinePtr tline[3];										//memory buffers for binary values
	BLinePtr rline;
	Image_ID	Temp1, Temp2;								//two temporary image memories
	long	odderase[256], evenerase[256], cleanup[256];	//fate tables
	// neighbor patterns for erasure on even and odd passes, and final cleanup pass
	long	odd[34] =  {3,6,7,12,14,15,24,28,30,48,56,60,62,96,112,120,129,
			131,135,143,192,193,195,199,207,224,225,227,231,240,241,243,248,249};
	long	even[34] = {3,6,7,12,14,15,24,28,30,31,48,56,60,62,63,96,112,120,
			124,126,129,131,135,143,159,192,193,195,224,225,227,240,248,252};
	long	clean[14] = {5,13,20,21,22,23,29,52,54,80,84,88,92,116};

	for (i = 0; i < 256; i++)
	{	odderase[i] = evenerase[i] = cleanup[i] = 1; // default = keep the pixel on
	}
	for (i = 0; i < 34; i++) // zero means erase
	{	odderase[odd[i]] = 0;
		evenerase[even[i]] = 0;
	}
	for (i = 0; i < 14; i++)
		cleanup[clean[i]] = 0;

	GetOriginalDimensions(&width, &height);		//find out how big the image is

	result = MakeTemporaryImage(width+2, height+2, sizeof(long),"T1", &Temp1);
	result = MakeTemporaryImage(width+2, height+2, sizeof(long),"T2", &Temp2);
		//Allocate temporary image workspaces one pixel larger on all sides
							
	line = CreateAPointer(width, sizeof(RGBPixel));			//for the original image
	for (i = 0; i < 3; i++)
		tline[i] = CreateAPointer(width+2, sizeof(long));	//for reading the temp image buffer
	rline = CreateAPointer(width+2, sizeof(long));			//and writing back

	for (x = 0; x < width+2; x++) // initializeline  array
		rline[x] = 0;

	WriteTempImageLine(Temp1, 0, rline);	//and edges of temporary arrays
	WriteTempImageLine(Temp1, height+1, rline);
	WriteTempImageLine(Temp2, 0, rline);
	WriteTempImageLine(Temp2, height+1, rline);

	// first pass - read in brightness. store in temp array
	for (y = 0; y < height; y++)
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadOriginalLine(y, line);
		for (x = 0; x < width; x++)
		{	//if pixel isn't black treat it as white
			brite = (line[x].red+line[x].green+line[x].blue)/3;
			if (brite > 0.5) rline[x+1] = 1;
			else rline[x+1] = 0; // binary values
		}
		WriteTempImageLine(Temp1, y+1, rline);	//notice offset by 1 in horizontal and vertical position
	}

	do
	{
		changes = FALSE;
		for (y = 1; y < height+1; y++) // odd pass
		{
			ReadTempImageLine(Temp1, y-1, tline[0]);//read three lines
			ReadTempImageLine(Temp1, y  , tline[1]);
			ReadTempImageLine(Temp1, y+1, tline[2]);
			
			for (x = 1; x < width+1; x++)
			{
				if (tline[1][x] > 0)
				{
					index = 0;//construct index based on neighbor pixels
					if (tline[0][x  ] > 0) index += 1;
					if (tline[0][x+1] > 0) index += 2;
					if (tline[1][x+1] > 0) index += 4;
					if (tline[2][x+1] > 0) index += 8;
					if (tline[2][x  ] > 0) index += 16;
					if (tline[2][x-1] > 0) index += 32;
					if (tline[1][x-1] > 0) index += 64;
					if (tline[0][x-1] > 0) index += 128;
					if (odderase[index] == 0) //if table says to, erase it
					{	rline[x] = 0;
						changes = TRUE;
					}
					else rline[x] = 1;
				}
				else rline[x] = 0;
			}
			WriteTempImageLine(Temp2, y, rline);
		}// for y

		//  even pass, same as above but from temp2 to temp1
		for (y = 1; y < height+1; y++)
		{
			ReadTempImageLine(Temp2, y-1, tline[0]);
			ReadTempImageLine(Temp2, y  , tline[1]);
			ReadTempImageLine(Temp2, y+1, tline[2]);
			
			for (x = 1; x < width+1; x++)
			{
				if (tline[1][x] > 0)
				{
					index = 0;
					if (tline[0][x  ] > 0) index += 1;
					if (tline[0][x+1] > 0) index += 2;
					if (tline[1][x+1] > 0) index += 4;
					if (tline[2][x+1] > 0) index += 8;
					if (tline[2][x  ] > 0) index += 16;
					if (tline[2][x-1] > 0) index += 32;
					if (tline[1][x-1] > 0) index += 64;
					if (tline[0][x-1] > 0) index += 128;
					if (evenerase[index] == 0) // erase it
					{
						rline[x] = 0;
						changes = TRUE;
					}
					else rline[x] = 1;
				}
				else rline[x] = 0;
			}
			WriteTempImageLine(Temp1, y, rline);
		}// for y
	}while (changes);

	// cleanup pass, make skeleton 8-neighbor connected everywhere
	do
	{
	 changes = FALSE;
		for (y = 1; y < height+1; y++)
		{
			ReadTempImageLine(Temp1, y-1, tline[0]);
			ReadTempImageLine(Temp1, y  , tline[1]);
			ReadTempImageLine(Temp1, y+1, tline[2]);
			
			for (x = 1; x < width+1; x++)
			{
				if (tline[1][x] > 0)
				{
					index = 0;
					if (tline[0][x  ] > 0) index += 1;
					if (tline[0][x+1] > 0) index += 2;
					if (tline[1][x+1] > 0) index += 4;
					if (tline[2][x+1] > 0) index += 8;
					if (tline[2][x  ] > 0) index += 16;
					if (tline[2][x-1] > 0) index += 32;
					if (tline[1][x-1] > 0) index += 64;
					if (tline[0][x-1] > 0) index += 128;
					if (cleanup[index] == 0) // erase it
					{
						rline[x] = 0;
						changes = TRUE;
					}
					else rline[x] = 1;
				}
				else rline[x] = 0;
			}
			WriteTempImageLine(Temp2, y, rline);
		}// for y
		
		for (y = 1; y < height+1; y++)
		{
			ReadTempImageLine(Temp2, y, rline);//copy back to temp image 1 for next iteration
			WriteTempImageLine(Temp1, y, rline);
		}
	}while (changes);

	// last pass - write it back
	for (y = 0; y < height; y++)
	{
		RGBPixel	WhitePix, BlackPix;
		WhitePix.red = WhitePix.green = WhitePix.blue = 255;
		BlackPix.red = BlackPix.green = BlackPix.blue = 0;

		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadTempImageLine(Temp1, y+1, rline);
		for (x = 0; x < width; x++)
		{	if (rline[x+1] == 0) line[x] = BlackPix;
			else line[x] = WhitePix;
		}
		WriteResultLine(y, line);
	}

mainexit:		//clean up everything we allocated
	if (line)
		DisposeAPointer(line);
	if (rline)
		DisposeAPointer(rline);
	for (i = 0; i < 3; i++)
		if (tline[i]) DisposeAPointer(tline[i]);
	// no need to dispose of the temp image arrays??

	return result;	//pass any errors back
}//MainUserEntry

