
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 4.3.4.6 Outline */

#include "PhotoshopShell.h"

typedef	RGBPixel *LinePtr; // could do this with short or even char values
///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	x, y, i;										//loop counter to traverse image
	long	NborArray[4];									//four edge-sharing neighbors
	long	total;											//counter
	LinePtr Line[3];										//memory buffers to read
	LinePtr Result;											//and write

	GetOriginalDimensions(&width, &height);					//find out how big the image is
							
	for (i = 0; i < 3; i++)
		Line[i] = CreateAPointer(width, sizeof(RGBPixel));	//for reading the temp image buffer
	Result = CreateAPointer(width, sizeof(RGBPixel));		//and writing back

	for (y = 1; y < height-1; y++)
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadOriginalLine(y-1, Line[0]);//read three line neighborhood
		ReadOriginalLine(y  , Line[1]);
		ReadOriginalLine(y+1, Line[2]);
		for (x = 1; x < width-1; x++)
		{
			float brite;//average of r, g, b is either 0=black or else assumed white
			brite = (Line[1][x  ].red + Line[1][x  ].green + Line[1][x  ].blue)/3;
			if (brite > 0.5)//if pixel is white
			{	//get the four edge-sharing neighbors 
				brite = (Line[0][x  ].red + Line[0][x  ].green + Line[0][x  ].blue)/3;
				if (brite > 0.5) NborArray[0] = 1; else NborArray[0] = 0;
				brite = (Line[1][x-1].red + Line[1][x-1].green + Line[1][x-1].blue)/3;
				if (brite > 0.5) NborArray[1] = 1; else NborArray[1] = 0;
				brite = (Line[1][x+1].red + Line[1][x+1].green + Line[1][x+1].blue)/3;
				if (brite > 0.5) NborArray[2] = 1; else NborArray[2] = 0;
				brite = (Line[2][x  ].red + Line[2][x  ].green + Line[2][x  ].blue)/3;
				if (brite > 0.5) NborArray[3] = 1; else NborArray[3] = 0;
				
				total = 0;
				for (i = 0; i < 4; i++)// count four edge-sharing neighbors that are white
					total += NborArray[i];
				if (total == 4) Result[x].red = Result[x].green = Result[x].blue = 0; // erase interior
				else Result[x].red = Result[x].green = Result[x].blue = 255; // keep boundary
			}
			else Result[x] = Line[1][x];
		}
		Result[0] = Line[1][0]; Result[width-1] = Line[1][width-1];
		WriteResultLine(y, Result);
	}// for y

mainexit:													//clean up everything we allocated
	DisposeAPointer(Result);
	for (i = 0; i < 3; i++)
		DisposeAPointer(Line[i]);

	return result;											//pass any errors back
}//MainUserEntry

