
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 4.4.4.1 Boolean Combination */

#include "PhotoshopShell.h"

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	rheight, rwidth;								//and reference image
	long	x, y;											//loop counter to traverse image
	float	brite;											//average of R, G, B values
	long	bRef, bOrig;									//binary black/white status of pixel in orig and ref images
	long	invertOrig = 0, invertRef = 0, Operator = 1;	//desired operation to perform
	// invert flag = 1 to invert, Operator code 1=AND 2=OR 3=XOR
	long fref;												//file to read operation code
	RGBPixel *oline = NULL, *rline = NULL;					//memory buffers for original and ref images

	GetOriginalDimensions(&width, &height);					//find out how big the image is
	GetRefDimensions(&rwidth, &rheight);					//find out how big the current ref image is

	if ((height != rheight) || (width != rwidth))
	{
		ErrorMessage("Dimensions of current and reference images do not match.");
		return (userCanceledErr);
	}

	if (OpenTextFileToRead("C:\\boolean.txt", &fref) != noErr) // find text file
	{
		result = fnfErr;
		goto mainexit;
	}
	else
	{
		float	dummy; // and read operation codes from the text file
		result = ReadANumber(fref, &dummy); invertOrig = (long)dummy;
		result = ReadANumber(fref, &dummy); invertRef = (long)dummy;
		result = ReadANumber(fref, &dummy); Operator = (long)dummy;
		CloseTextFile(fref);
		if (result != noErr)
			goto mainexit;
	}

	oline = CreateAPointer(width, sizeof(RGBPixel));	//Allocate memory for one row from either image
	rline = CreateAPointer(width, sizeof(RGBPixel));

	for (y = 0; y < height; y++)//line by line
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadRefImageLine(y, rline);//read reference image
		ReadOriginalLine(y, oline);//and current image
		for (x = 0; x < width; x++)
		{
			brite = (oline[x].red + oline[x].green + oline[x].blue)/3;// convert original pixel to binary
			if (brite < 0.5) bOrig = 0; else bOrig = 1;
			if (invertOrig == 1) bOrig = 1 - bOrig;// and optionally invert it
			brite = (rline[x].red + rline[x].green + rline[x].blue)/3;// convert reference pixel to binary
			if (brite < 0.5) bRef = 0; else bRef = 1;
			if (invertRef == 1) bRef = 1 - bRef;// and optionally invert it
			
			switch (Operator)// which Boolean operator was selected
			{	case 1: // AND
				{	if ((bRef == 1) && (bOrig == 1)) brite = 255; else brite = 0;
					break;
				}
				case 2: // OR
				{	if ((bRef == 1) || (bOrig == 1)) brite = 255; else brite = 0;
					break;
				}
				case 3: // XOR
				{	if (bRef != bOrig) brite = 255; else brite = 0;
					break;
				}
			}
			oline[x].red = oline[x].green = oline[x].blue = brite;//write result pixel back
		}
		WriteResultLine(y, oline);
	}
mainexit:									//clean up everything we allocated
	if (oline)
		DisposeAPointer(oline);
	if (rline)
		DisposeAPointer(rline);

	return result;							//pass any errors back
}//MainUserEntry

