
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 5.1.5.1 Global Measurements */

#include "PhotoshopShell.h"

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	x, y, val;										//loop counters for image, pixel value
	long	area = 0, index;								//sum area
	float	perim = 0;										//and perimeter
	float	brite;											//RGB pixel brightness
	long	nbr1, nbr2, nbr4, nbr8;							//four neighbors above and left
	float	ONTable[16] = {0, (float)1.414, 1, 1,			//tables with perimeter contributions to sum
		(float)2.828, (float)4.243, 	
		(float)1.414, (float)1.414, 1, (float)2.414, 0, 0, 
		(float)1.414, (float)2.828, 0, 0};
	float	OFFTable[16] = {0, 0, 0, 0, 0, 0, 1, 1, 0, 0,
		(float)1.414, 0, 1, 0, (float)1.414, (float)1.414};
	long fref;												//file to write results
	RGBPixel *OrigLine = NULL;								//memory buffer for RGB pixels
	long *ThisLine = NULL, *PrevLine = NULL;				//memory buffers for current line and line above
	Image_ID	Bin;										//temp image 

	GetOriginalDimensions(&width, &height);					//find out how big the image is

	OrigLine = CreateAPointer(width, sizeof(RGBPixel));		//Allocate memory for one row of RGB pixels
	ThisLine = CreateAPointer(width+2, sizeof(long));		//and for two lines of binary pixels
	PrevLine = CreateAPointer(width+2, sizeof(long));

	result = MakeTemporaryImage(width+2, height+2, sizeof(long),"Binary", &Bin); // padded on all sides

	for(x = 0; x < width+2; x++) // initialize lines and array
		ThisLine[x] = 0;
	WriteTempImageLine(Bin, 0, ThisLine);
	WriteTempImageLine(Bin, height+1, ThisLine);
	
	// copy image to binary temp buffer
	//(could do this directly on RGB pixels but binary image is useful in future problems as well)
	for (y = 0; y < height; y++)
	{
		ReadOriginalLine(y, OrigLine);
		for (x = 0; x < width; x++)
		{	//read pixel brightness, treat as either black or white
			brite = (OrigLine[x].red + OrigLine[x].green + OrigLine[x].blue)/3;
			if (brite < 0.5) ThisLine[x+1] = 0; else ThisLine[x+1] = 1;
		}
		WriteTempImageLine(Bin, y+1, ThisLine);//write temporary image
	}

	for (y = 1; y < height+1; y++)
	{	//scan through image line by line
		ReadTempImageLine(Bin, y-1, PrevLine);//line above
		ReadTempImageLine(Bin, y  , ThisLine);//current line
		for (x = 1; x < width+1; x++)//for each pixel
		{
			index = 0; // construct index based on 4 neighbors above and left
			val = ThisLine[x];
			nbr8 = ThisLine[x-1]; if (nbr8 != 0) index += 8;
			nbr4 = PrevLine[x-1]; if (nbr4 != 0) index += 4;
			nbr2 = PrevLine[x  ]; if (nbr2 != 0) index += 2;
			nbr1 = PrevLine[x+1]; if (nbr1 != 0) index += 1;
			
			if (val != 0) perim += ONTable[index];//sum perimeter depending on whether current pixel is on
			else perim += OFFTable[index];//or off
			if (val != 0) area ++;//sum area
		}// for x
	}

	{	// write the data to a text file
		if (CreateTextFileToWrite("C:\\global.txt", &fref) == 0);//location to write file
		{
			WriteANumber(fref, (float)area);
			WriteANumber(fref, perim);
		}// if file was successfully opened
		CloseTextFile(fref);
	}
	
	//clean up everything we allocated
	DisposeAPointer(OrigLine);
	DisposeAPointer(ThisLine);
	DisposeAPointer(PrevLine);

	return result;											//pass any errors back
}//MainUserEntry

