
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 5.1.5.2 Count Features */

#include "PhotoshopShell.h"

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	x, y, val, i, j;								//loop counters for image and neighborhood, pixel id value
	long	number = 0, index, featID = 1;					//neighbor index and feature count and label
	float	brite;											//average RGB pixel brightness
	long	newval, oldval, nbr1, nbr2, nbr4, nbr8;			//neighbor id values
	RGBPixel *OrigLine = NULL;								//memory buffer for RGB pixels
	long *ThisLine = NULL, *PrevLine = NULL, *TempLine = NULL;//memory buffers for current line, line above, result line
	Image_ID	Bin;										//pointer for temporary binary image memory
	long fref;												//file to write results

	GetOriginalDimensions(&width, &height);					//find out how big the image is

	OrigLine = CreateAPointer(width, sizeof(RGBPixel));		//Allocate memory for one row from RGB image
	ThisLine = CreateAPointer(width+2, sizeof(long));		//and three lines to access binary image
	PrevLine = CreateAPointer(width+2, sizeof(long));
	TempLine = CreateAPointer(width+2, sizeof(long));

	result = MakeTemporaryImage(width+2, height+2, sizeof(long),"Binary", &Bin); // padded on all sides

	for(x = 0; x < width+2; x++)	// initialize
		ThisLine[x] = 0; // border pixels
	WriteTempImageLine(Bin, 0, ThisLine);
	WriteTempImageLine(Bin, height+1, ThisLine);
	
	// copy image to temp buffer
	for (y = 0; y < height; y++)
	{
		ReadOriginalLine(y, OrigLine);
		for (x = 0; x < width; x++)
		{	//if pixel is not black treat it as white, build temp binary image
			brite = (OrigLine[x].red + OrigLine[x].green + OrigLine[x].blue)/3;
			if (brite < 0.5) ThisLine[x+1] = 0; else ThisLine[x+1] = 1;
		}
		WriteTempImageLine(Bin, y+1, ThisLine);
	}

	for (y = 1; y < height+1; y++)//scan line by line through binary image
	{
		ReadTempImageLine(Bin, y-1, PrevLine);//line above
		ReadTempImageLine(Bin, y  , ThisLine);//current line
		for (x = 1; x < width+1; x++)//for each pixel
		{
			index = 0;//construct index number from four neighbors above and left
			val = ThisLine[x];
			nbr8 = ThisLine[x-1]; if (nbr8 != 0) index += 8;
			nbr4 = PrevLine[x-1]; if (nbr4 != 0) index += 4;
			nbr2 = PrevLine[x  ]; if (nbr2 != 0) index += 2;
			nbr1 = PrevLine[x+1]; if (nbr1 != 0) index += 1;
			if (val != 0) // pixel is white, so perform feature counting
			{	if (index == 0) // new - count and assign ID
				{	number ++;
					featID ++;
					newval = featID;
				}
				// propagate id from previous pixel
				else if ((index == 1) || (index == 3) || (index == 7) || (index == 11) || (index == 15))
					newval = nbr1;
				else if ((index == 2) || (index == 6) || (index == 10) || (index == 14))
					newval = nbr2;
				else if ((index == 4) || (index == 12)) 
					newval = nbr4;
				else if (index == 8)
					newval = nbr8;
				else // test for merging id numbers
				{	newval = nbr1;
					if ((index == 9) || (index == 13)) oldval = nbr8;
					else oldval = nbr4;
					if (oldval != newval)
					{	// change id in prior lines
						number --; // and decrement feature count
						for (i = 1; i <= y; i ++)
						{	// rewrite lines above
							ReadTempImageLine(Bin, i, TempLine);
							for (j = 1; j < width+1; j++)
								if (TempLine[j] == oldval) TempLine[j] = newval;
							WriteTempImageLine(Bin, i, TempLine);
						}
					}
					for (j = 1; j < x; j++) // also change current line
						if (ThisLine[j] == oldval) ThisLine[j] = newval;
					ReadTempImageLine(Bin, y-1, PrevLine); // reload changed line
				}
				ThisLine[x] = newval;// write feature id number to pixel in binary image
			}//counting
		}// for x
		WriteTempImageLine(Bin, y, ThisLine);//rewrite line to binary
	}// for y

	{	// write the number to a text file
		if (CreateTextFileToWrite("C:\\count.txt", &fref) == 0);//location to write file
		{
			WriteANumber(fref, (float)number);
		}// if file was successfully opened
		CloseTextFile(fref);
	}

	if (featID < 255) // display feature numbering as grey scale
	for (y = 0; y < height; y++)
	{
		ReadTempImageLine(Bin, y+1, ThisLine);
		for (x = 0; x < width; x++)
		{	// assign a grey scale value to the pixels in the feature
			if (ThisLine[x+1] == 0) brite = 0.0;
			else brite = (float)255.0 - ThisLine[x+1];
			OrigLine[x].red = OrigLine[x].green = OrigLine[x].blue = brite;
		}// for x
		WriteResultLine(y, OrigLine);
	}//for y 

	//clean up everything we allocated
	DisposeAPointer(OrigLine);
	DisposeAPointer(ThisLine);
	DisposeAPointer(PrevLine);
	DisposeAPointer(TempLine);

	return result;											//pass any errors back
}//MainUserEntry

