
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 5.2.5.1 Feature Length */

#include "PhotoshopShell.h"

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	x, y, val, i, j;								//loop variables and pixel id
	long	number = 0, index, count, featID = 1, New_ID = 0;//feature count and id
	float	brite;											//mean rgb pixel brightness
	long	newval, oldval, nbr1, nbr2, nbr4, nbr8;			//neighbor ids
	RGBPixel *OrigLine = NULL;								//memory pointer for rgb pixels
	long *ThisLine = NULL, *PrevLine = NULL, *TempLine = NULL;//memory buffers for binary image
	float	*len;											// array to hold result values
	Image_ID	Bin;										//temp binary image

	GetOriginalDimensions(&width, &height);					//find out how big the image is

	OrigLine = CreateAPointer(width, sizeof(RGBPixel));		//Allocate memory for one row from either image
	ThisLine = CreateAPointer(width+2, sizeof(long));	
	PrevLine = CreateAPointer(width+2, sizeof(long));
	TempLine = CreateAPointer(width+2, sizeof(long));

	result = MakeTemporaryImage(width+2, height+2, sizeof(long),"Binary", &Bin); // padded on all sides

	for(x = 0; x < width+2; x++)
		ThisLine[x] = 0; // border pixels
	WriteTempImageLine(Bin, 0, ThisLine);
	WriteTempImageLine(Bin, height+1, ThisLine);
	
	// copy image to temp buffer
	for (y = 0; y < height; y++)
	{	if (TestAbortSetProgressBar((float)y/(float)(height*2)))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadOriginalLine(y, OrigLine);
		for (x = 0; x < width; x++)
		{	
			brite = (OrigLine[x].red + OrigLine[x].green + OrigLine[x].blue)/3;
			if (brite < 0.5) ThisLine[x+1] = 0; else ThisLine[x+1] = 1;
		}
		WriteTempImageLine(Bin, y+1, ThisLine);
	}

	// feature identification
	for (y = 1; y < height+1; y++)
	{	
		ReadTempImageLine(Bin, y-1, PrevLine);
		ReadTempImageLine(Bin, y  , ThisLine);
		for (x = 1; x < width+1; x++)
		{	
			index = 0;
			val = ThisLine[x];
			nbr8 = ThisLine[x-1]; if (nbr8 != 0) index += 8;
			nbr4 = PrevLine[x-1]; if (nbr4 != 0) index += 4;
			nbr2 = PrevLine[x  ]; if (nbr2 != 0) index += 2;
			nbr1 = PrevLine[x+1]; if (nbr1 != 0) index += 1;
			if (val != 0) // feature counting
			{	if (index == 0) // new - count and assign ID
				{	
					number ++;
					featID ++;
					newval = featID;
				}
				// propagate id from previous pixel
				else if ((index == 1) || (index == 3) || (index == 7) || (index == 11) || (index == 15))
					newval = nbr1;
				else if ((index == 2) || (index == 6) || (index == 10) || (index == 14))
					newval = nbr2;
				else if ((index == 4) || (index == 12)) 
					newval = nbr4;
				else if (index == 8)
					newval = nbr8;
				else // test for merging id numbers
				{	
					newval = nbr1;
					if ((index == 9) || (index == 13)) oldval = nbr8;
					else oldval = nbr4;
					if (oldval != newval)
					{	// change id in prior lines
						number --; // decrement feature count
						for (i = 1; i <= y; i ++)
						{	
							ReadTempImageLine(Bin, i, TempLine);
							for (j = 1; j < width+1; j++)
								if (TempLine[j] == oldval) TempLine[j] = newval;
							WriteTempImageLine(Bin, i, TempLine);
						}
					}
					for (j = 1; j < x; j++) // also change current line
						if (ThisLine[j] == oldval) ThisLine[j] = newval;
					ReadTempImageLine(Bin, y-1, PrevLine); // reload changed line
				}
				ThisLine[x] = newval;
			}//counting
		}// for x
		WriteTempImageLine(Bin, y, ThisLine);
	}// for y

	// renumber consecutively and erase edge-touching features
	{	
		Boolean	found, edge;
		for (count = 1; count <= featID; count++)
		{	
			found = FALSE; //is this number used?
			edge = FALSE;  // does it touch any edge?
			for (y = 1; y < height+1; y++)
			{	
				ReadTempImageLine(Bin, y, ThisLine);
				for (x = 1; x < width+1; x++)
				{	
					if (ThisLine[x] == count)
					{	
						if ((x==1) || (x==width) || (y==1) || (y==height))
							edge = TRUE;
						else
							found = TRUE;
					}
				}// for x
			}// for y
			if (edge)
			{	
				for (y = 1; y < height+1; y++)
				{	
					ReadTempImageLine(Bin, y, ThisLine);
					for (x = 1; x < width+1; x++)
					{	if (ThisLine[x] == count) ThisLine[x] = 0;
					}
					WriteTempImageLine(Bin, y, ThisLine);
				}
			} // erase the edge-touching feature
			else if (found)
			{	
				New_ID++;
				for (y = 1; y < height+1; y++)
				{	
					ReadTempImageLine(Bin, y, ThisLine);
					for (x = 1; x < width+1; x++)
					{	
						if (ThisLine[x] == count) ThisLine[x] = New_ID;
					}
					WriteTempImageLine(Bin, y, ThisLine);
				}
				
			} //renumber consecutively
		}// for count
	}//renumber section

	// find rotated xprimemin and max for each feature
	{	float	xmin[8], xmax[8];//8 directions
		long	a;			//loop counter for directions
		long	fref = 0;	//file to write results
		float	cosine[8], sine[8];//table of sines and cosines
		float	xprime, length;	// length calculation
		
		for (a = 0; a < 8; a++) // build tables (could calculate them on the fly, this is faster)
		{	
			sine[a] = (float)sin(a*22.5*3.14159265/180.0);
			cosine[a] = (float)cos(a*22.5*3.14159265/180.0);
		}
		len = CreateAPointer(New_ID+1, sizeof(float)); // now we know how many we need
		CreateTextFileToWrite("C:\\lengths.txt", &fref);//location to write file

		for (count = 1; count <= New_ID; count++) // perform calculation for each feature ID
		{
			for (a = 0; a < 8; a++) // for each direction
			{	
				xmin[a] = 10000; xmax[a] = -10000; // initialize
			}
			for (y = 1; y < height+1; y++) // scan through image
			{
				ReadTempImageLine(Bin, y, ThisLine); // read line
				for (x = 1; x < width+1; x++) // see if any pixels are in this feature
				{
					if (ThisLine[x] == count)
					{
						for (a = 0; a < 8; a++) // if so get all rotated coordinates
						{
							xprime = (float)x * cosine[a] + (float)y * sine[a];
							if (xprime < xmin[a]) xmin[a] = xprime; // and find min, max
							if (xprime > xmax[a]) xmax[a] = xprime;
						}
					}
				}// for x
			}// for y
			length = xmax[0]-xmin[0]; index = 0; // see which direction has greatest difference
			for (a = 1; a < 8; a++)
			{
				float temp = xmax[a] - xmin[a];
				if (temp>length)
				{	
					length = temp;
					index = a;
				}
			}
			len[count] = length; // that value is the caliper length - save it to use below
			if (fref != 0) // if file was successfully opened
			{	
				WriteANumber(fref, (float)count);//write out the id number and the length value
				WriteANumber(fref, len[count]);
			}
		}//for count
		if (fref != 0) CloseTextFile(fref);
	} //measurement section

	// find limits for measured parameter and display results
	{
		float minv = (float)1e20, maxv = (float)-1e20, value;
		for (index = 1; index <= New_ID; index ++)
		{
			if (len[index] < minv) minv = len[index]; // get shortest and longest limits
			if (len[index] > maxv) maxv = len[index];
		}
		if (maxv>minv) // display feature values as hues
		{
			for (y = 0; y < height; y++)//read through binary image line by line
			{
				if (TestAbortSetProgressBar((float)y/(float)height))
				{// test for cancel and advance progress bar
					result = userCanceledErr;
					goto mainexit;
				}
				ReadTempImageLine(Bin, y+1, ThisLine);
				for (x = 0; x < width; x++) // and pixel by pixel
				{
					if (ThisLine[x+1] == 0) // if not a feature, set RGB pixel to black
					{	
						OrigLine[x].red = OrigLine[x].green = OrigLine[x].blue  = 0.0;
					}	
					else // use ID to look up length value
					{	
						index = ThisLine[x+1];
						value = len[index];
						value = (value - minv) / (maxv - minv); // 0..1 range
						// and convert scaled value to a hue
						if (value > 0.75) // red to yellow
						{	
							OrigLine[x].red = 255.0;
							OrigLine[x].green = (float)1020.0 * ((float)1.0 - value);
							OrigLine[x].blue = 0.0;
						}
						else if (value > 0.55) // yellow to green
						{	
							OrigLine[x].red = (float)1275 * (value - (float)0.55);
							OrigLine[x].green = 255.0;
							OrigLine[x].blue = 0.0;
						}
						else if (value > 0.4) // green to cyan
						{	
							OrigLine[x].red = 0.0;
							OrigLine[x].green = 255.0;
							OrigLine[x].blue = (float)1700 * ((float)0.55 - value);
						}
						else if (value > 0.2) // cyan to blue
						{	
							OrigLine[x].red = 0.0;
							OrigLine[x].green = (float)1275 * (value - (float)0.2);
							OrigLine[x].blue = 255.0;
						}
						else // blue to magenta
						{	
							OrigLine[x].red = (float)1275 * ((float)0.2 - value);
							OrigLine[x].green = 0.0;
							OrigLine[x].blue = 255.0;
						}
					// cheap and cheerful conversion from value(0..1) to hue (blue..red)
					}// color feature
				}// for x
				WriteResultLine(y, OrigLine);
			}//for y 
		}
	}// display section

mainexit:				//clean up everything we allocated
	DisposeAPointer(OrigLine);
	DisposeAPointer(ThisLine);
	DisposeAPointer(PrevLine);
	DisposeAPointer(TempLine);
	DisposeAPointer(len);

	return result;		//pass any errors back
}//MainUserEntry

