
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 5.2.5.5 Feature Location */

#include "PhotoshopShell.h"

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{	Boolean	UseCentroid = TRUE;							//choose between centroid and bounding box location
	ErrType result = noErr;								//accumulate any errors that happened
	long	height, width;								//pixel dimensions of image
	long	x, y, val, i, j;							//loop counters, pixel ID
	long	number = 0, index, count, featID = 1;		//feature counting and ID
	float	brite;										//mean rgb pixel brightness
	long	newval, oldval, nbr1, nbr2, nbr4, nbr8;		//neighbor pixel id values
	RGBPixel *OrigLine = NULL;							//pointer to line of rgb pixels
	long *ThisLine = NULL, *PrevLine = NULL, *TempLine = NULL;//pointers to lines of binary image pixels
	Image_ID	Bin;									//temporary binary image with feature id

	GetOriginalDimensions(&width, &height);					//find out how big the image is

	OrigLine = CreateAPointer(width, sizeof(RGBPixel));		//Allocate memory for one row from either image
	ThisLine = CreateAPointer(width+2, sizeof(long));	
	PrevLine = CreateAPointer(width+2, sizeof(long));
	TempLine = CreateAPointer(width+2, sizeof(long));

	result = MakeTemporaryImage(width+2, height+2, sizeof(long),"Binary", &Bin); // padded on all sides

	for(x = 0; x < width+2; x++)
		ThisLine[x] = 0; // border pixels
	WriteTempImageLine(Bin, 0, ThisLine);
	WriteTempImageLine(Bin, height+1, ThisLine);
	
	// copy image to temp buffer
	for (y = 0; y < height; y++)
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadOriginalLine(y, OrigLine);
		for (x = 0; x < width; x++)
		{	//create temp binary image, 0 for no feature, 1 for feature
			brite = (OrigLine[x].red + OrigLine[x].green + OrigLine[x].blue)/3;
			if (brite < 0.5) ThisLine[x+1] = 0; else ThisLine[x+1] = 1;
		}
		WriteTempImageLine(Bin, y+1, ThisLine);//note offset +1 in x and y
	}

	// feature identification
	for (y = 1; y < height+1; y++)
	{	ReadTempImageLine(Bin, y-1, PrevLine);
		ReadTempImageLine(Bin, y  , ThisLine);
		for (x = 1; x < width+1; x++)
		{	index = 0;
			val = ThisLine[x];
			nbr8 = ThisLine[x-1]; if (nbr8 != 0) index += 8;
			nbr4 = PrevLine[x-1]; if (nbr4 != 0) index += 4;
			nbr2 = PrevLine[x  ]; if (nbr2 != 0) index += 2;
			nbr1 = PrevLine[x+1]; if (nbr1 != 0) index += 1;
			if (val != 0) // feature counting
			{	if (index == 0) // new - count and assign ID
				{	number ++;
					featID ++;
					newval = featID;
				}
				// propagate id from previous pixel
				else if ((index == 1) || (index == 3) || (index == 7) || (index == 11) || (index == 15))
					newval = nbr1;
				else if ((index == 2) || (index == 6) || (index == 10) || (index == 14))
					newval = nbr2;
				else if ((index == 4) || (index == 12)) 
					newval = nbr4;
				else if (index == 8)
					newval = nbr8;
				else // test for merging id numbers
				{	newval = nbr1;
					if ((index == 9) || (index == 13)) oldval = nbr8;
					else oldval = nbr4;
					if (oldval != newval)
					{	// change id in prior lines
						number --; // decrement feature count
						for (i = 1; i <= y; i ++)
						{	ReadTempImageLine(Bin, i, TempLine);
							for (j = 1; j < width+1; j++)
								if (TempLine[j] == oldval) TempLine[j] = newval;
							WriteTempImageLine(Bin, i, TempLine);
						}
					}
					for (j = 1; j < x; j++) // also change current line
						if (ThisLine[j] == oldval) ThisLine[j] = newval;
					ReadTempImageLine(Bin, y-1, PrevLine); // reload changed line
				}
				ThisLine[x] = newval;
			}//counting
		}// for x
		WriteTempImageLine(Bin, y, ThisLine);
	}// for y

	// erase edge-touching features
	{	Boolean	edge;
		for (count = 1; count <= featID; count++)
		{	edge = FALSE;  // does it touch any edge?
			ReadTempImageLine(Bin, 1, ThisLine); // top boundary
			for (x = 1; x < width+1; x++)
				if (ThisLine[x] == count) edge = TRUE;
			ReadTempImageLine(Bin, height, ThisLine); // bottom boundary
			for (x = 1; x < width+1; x++)
				if (ThisLine[x] == count) edge = TRUE;
			for (y = 1; y < height+1; y++)
			{	ReadTempImageLine(Bin, y, ThisLine);
				if ((ThisLine[1] == count) || (ThisLine[width] == count))
					edge = TRUE;
			}// for y
			if (edge)
			{	for (y = 1; y < height+1; y++)
				{	ReadTempImageLine(Bin, y, ThisLine);
					for (x = 1; x < width+1; x++)
					{	if (ThisLine[x] == count) ThisLine[x] = 0;
					}
					WriteTempImageLine(Bin, y, ThisLine);
				}
			} // erase the edge-touching feature
		}// for count
	}
	if (!UseCentroid) // measure center of bounding box
	{	long	fref = 0; // file to write results
		long	k = 0;	// counter for features
		long	minx, miny, maxx, maxy;

		CreateTextFileToWrite("C:\\position.txt", &fref);//location to write file
		for (count = 1; count <= featID; count++)// for each feature
		{
			minx = miny = 10000; maxx = maxy = 0;//min and max x, y (box limits)
			for (y = 1; y < height+1; y++)//read binary image line by line
			{
				ReadTempImageLine(Bin, y, ThisLine);
				for (x = 1; x < width+1; x++)
				{	if (ThisLine[x] == count)//if pixel is in this feature
					{	if (x > maxx) maxx = x; if (x < minx) minx = x;//keep min, max x, y
						if (y > maxy) maxy = y; if (y < miny) miny = y;
					}
				}// for x
			}
			if (maxx > 0) // this number is in use
			{	k++;
				if (fref != 0) // if file was successfully opened
				{	WriteANumber(fref, (float)k);
					WriteANumber(fref, (float)((minx + maxx)/2));
					WriteANumber(fref, (float)((miny + maxy)/2));
				}
				y = (miny + maxy)/2;
				x = (minx + maxx)/2;
				ReadTempImageLine(Bin, y+1, ThisLine);
				ThisLine[x+1] = -1; // mark centroid location
				WriteTempImageLine(Bin, y+1, ThisLine);
			}
		}//for count
		if (fref != 0) CloseTextFile(fref);
	} //measurement section
	
	else // measure feature centroid locations
	{	long	fref = 0;//output file
		long	k = 0;//counter
		float	sumx, sumy, area;//sums for centroid

		CreateTextFileToWrite("C:\\position.txt", &fref);//location to write file
		for (count = 1; count <= featID; count++)//for each feature id
		{
			sumx = sumy = area = 0.0;//init sums
			for (y = 1; y < height+1; y++)
			{	//read each line of binary image
				ReadTempImageLine(Bin, y, ThisLine);
				for (x = 1; x < width+1; x++)
				{	//for each pixel that is in the current feature
					if (ThisLine[x] == count)
					{	area += 1.0; // build the sums
						sumx += (float)(x-1);
						sumy += (float)(y-1);
					}
				}// for x
			}
			if (area > 0) // this number is in use
			{	sumx = sumx / area;	// centroid x and y
				sumy = sumy / area;
				k++;
				if (fref != 0) // if file was successfully opened
				{	WriteANumber(fref, (float)k); // write out
					WriteANumber(fref, sumx);
					WriteANumber(fref, sumy);
				}
				y = (long)(sumy+0.5);
				x = (long)(sumx+0.5);
				ReadTempImageLine(Bin, y+1, ThisLine);
				ThisLine[x+1] = -1; // and mark location in binary image
				WriteTempImageLine(Bin, y+1, ThisLine);
			}
		}//for count
		if (fref != 0) CloseTextFile(fref);
	} //measurement section

	//rewrite image with controids marked
	for (y = 0; y < height; y++)
	{
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadTempImageLine(Bin, y+1, ThisLine);//read binary
		for (x = 0; x < width; x++)
		{	j = ThisLine[x+1];
			if (j == -1)//mark centroid or center location in red
			{	OrigLine[x].red = 255;
				OrigLine[x].green = OrigLine[x].blue  = 0.0;
			}
			else if (j == 0)
			{	OrigLine[x].red = OrigLine[x].green = OrigLine[x].blue  =  0.0;//background in black
			}
			else
			{	OrigLine[x].red = OrigLine[x].green = OrigLine[x].blue  =  255.0;//features in white
			}
		}
		WriteResultLine(y, OrigLine);
	}//for y 

mainexit:				//clean up everything we allocated
	DisposeAPointer(OrigLine);
	DisposeAPointer(ThisLine);
	DisposeAPointer(PrevLine);
	DisposeAPointer(TempLine);

	return result;		//pass any errors back
}//MainUserEntry

