
/* Photoshop plug-in example */
/*
	Copyright (c) 1996-2007, J. Christian Russ and John C. Russ
	All rights reserved.

	This code may be used by purchasers of the book for educational and personal research
	purposes but not for any commercial purpose.  The Copyright notices must be preserved
	in all copies. Plug-ins utilizing or derived from this code may not be sold or 
	distributed for a fee.
*/
/* Problem 5.2.5.7 Measure Feature FormFactor */

#include "PhotoshopShell.h"

///////////////////////////////////
//
//	Main entry for user code
//
///////////////////////////////////
ErrType	MainUserEntry()	
{
	ErrType result = noErr;									//accumulate any errors that happened
	long	height, width;									//pixel dimensions of image
	long	x, y, val, i, j, feat;							//loop counters
	long	number = 0, index, count, featID = 1;			//feature ident
	float	brite;											//mean rgb brightness
	long	newval, oldval, nbr1, nbr2, nbr4, nbr8;			//neighbor id's for numbering
	RGBPixel *OrigLine = NULL;								//line of rgb pixels
	long *ThisLine = NULL, *PrevLine = NULL, *TempLine = NULL;//lines of binary pixels
	float	*ff;											//array to hold result values		
	long	*fnum;											// array to keep track of feature numbers
	Image_ID	Bin;										//temporary binary array with feature numbers

	GetOriginalDimensions(&width, &height);					//find out how big the image is

	OrigLine = CreateAPointer(width, sizeof(RGBPixel));		//Allocate memory for one row from either image
	ThisLine = CreateAPointer(width+2, sizeof(long));	
	PrevLine = CreateAPointer(width+2, sizeof(long));
	TempLine = CreateAPointer(width+2, sizeof(long));

	result = MakeTemporaryImage(width+2, height+2, sizeof(long),"Binary", &Bin); // padded on all sides

	for(x = 0; x < width+2; x++) // initialize array
		ThisLine[x] = 0; // border pixels
	WriteTempImageLine(Bin, 0, ThisLine);
	WriteTempImageLine(Bin, height+1, ThisLine);
	
	// copy image to temp binary image
	for (y = 0; y < height; y++)
	{	
		if (TestAbortSetProgressBar((float)y/(float)height))
		{// test for cancel and advance progress bar
			result = userCanceledErr;
			goto mainexit;
		}
		ReadOriginalLine(y, OrigLine);
		for (x = 0; x < width; x++)
		{	//treat black as background, anything else as feature
			brite = (OrigLine[x].red + OrigLine[x].green + OrigLine[x].blue)/3;
			if (brite < 0.5) ThisLine[x+1] = 0; else ThisLine[x+1] = 1;
		}
		WriteTempImageLine(Bin, y+1, ThisLine); // note +1 offsets in x and y
	}

	// feature identification
	for (y = 1; y < height+1; y++)
	{
		ReadTempImageLine(Bin, y-1, PrevLine);
		ReadTempImageLine(Bin, y  , ThisLine);
		for (x = 1; x < width+1; x++)
		{	
			index = 0;
			val = ThisLine[x];
			nbr8 = ThisLine[x-1]; if (nbr8 != 0) index += 8;
			nbr4 = PrevLine[x-1]; if (nbr4 != 0) index += 4;
			nbr2 = PrevLine[x  ]; if (nbr2 != 0) index += 2;
			nbr1 = PrevLine[x+1]; if (nbr1 != 0) index += 1;
			if (val != 0) // feature counting
			{	
				if (index == 0) // new - count and assign ID
				{	number ++;
					featID ++;
					newval = featID;
				}
				// propagate id from previous pixel
				else if ((index == 1) || (index == 3) || (index == 7) || (index == 11) || (index == 15))
					newval = nbr1;
				else if ((index == 2) || (index == 6) || (index == 10) || (index == 14))
					newval = nbr2;
				else if ((index == 4) || (index == 12)) 
					newval = nbr4;
				else if (index == 8)
					newval = nbr8;
				else // test for merging id numbers
				{	
					newval = nbr1;
					if ((index == 9) || (index == 13)) oldval = nbr8;
					else oldval = nbr4;
					if (oldval != newval)
					{	// change id in prior lines
						number --; // decrement feature count
						for (i = 1; i <= y; i ++)
						{	
							ReadTempImageLine(Bin, i, TempLine);
							for (j = 1; j < width+1; j++)
								if (TempLine[j] == oldval) TempLine[j] = newval;
							WriteTempImageLine(Bin, i, TempLine);
						}
					}
					for (j = 1; j < x; j++) // also change current line
						if (ThisLine[j] == oldval) ThisLine[j] = newval;
					ReadTempImageLine(Bin, y-1, PrevLine); // reload changed line
				}
				ThisLine[x] = newval;
			}//counting
		}// for x
		WriteTempImageLine(Bin, y, ThisLine);
	}// for y

	// erase edge-touching features
	{	Boolean	edge;
		for (count = 1; count <= featID; count++)
		{	
			edge = FALSE;  // does it touch any edge?
			ReadTempImageLine(Bin, 1, ThisLine); // top boundary
			for (x = 1; x < width+1; x++)
				if (ThisLine[x] == count) edge = TRUE;
			ReadTempImageLine(Bin, height, ThisLine); // bottom boundary
			for (x = 1; x < width+1; x++)
				if (ThisLine[x] == count) edge = TRUE;
			for (y = 1; y < height+1; y++)
			{	
				ReadTempImageLine(Bin, y, ThisLine);
				if ((ThisLine[1] == count) || (ThisLine[width] == count))
					edge = TRUE;
			}// for y
			if (edge)
			{	
				for (y = 1; y < height+1; y++)
				{	
					ReadTempImageLine(Bin, y, ThisLine);
					for (x = 1; x < width+1; x++)
						if (ThisLine[x] == count) 
							ThisLine[x] = 0;
					WriteTempImageLine(Bin, y, ThisLine);
				}
			} // erase the edge-touching feature
		}// for count
	}

	// measure features
	{	long	fref = 0;
		float	area, perim;
		float	ONTable[16] = {0, (float)1.414, 1, 1, (float)2.828, (float)4.243, 
			(float)1.414, (float)1.414, 1, (float)2.414, 0, 0, (float)1.414, (float)2.828, 0, 0};
		float	OFFTable[16] = {0, 0, 0, 0, 0, 0, 1, 1, 0, 0,
			(float)1.414, 0, 1, 0, (float)1.414, (float)1.414};

		ff = CreateAPointer(number+1, sizeof(float)); // array to hold results
		fnum = CreateAPointer(featID+1, sizeof(long));	//array to hold feature numbers
		feat = 0;
		CreateTextFileToWrite("C:\\formfactor.txt", &fref);//location to write file

		for (count = 1; count <= featID; count++) // for each feature number
		{
			area = perim = 0.0; // initialize sums
			for (y = 1; y < height+1; y++)
			{
				ReadTempImageLine(Bin, y-1, PrevLine); // read current and previous lines of binary image
				ReadTempImageLine(Bin, y  , ThisLine);
				for (x = 1; x < width+1; x++)// for each pixel
				{
					index = 0;
					val = ThisLine[x];
					nbr8 = ThisLine[x-1]; if (nbr8 != 0) index += 8;//build index number from upper/left nbors
					nbr4 = PrevLine[x-1]; if (nbr4 != 0) index += 4;
					nbr2 = PrevLine[x  ]; if (nbr2 != 0) index += 2;
					nbr1 = PrevLine[x+1]; if (nbr1 != 0) index += 1;
					if (val == count) // if pixel belongs to current feature
					{	
						perim += ONTable[index]; // add to perimeter and area
						area += 1.0;
					}
					else // if preceding pixel(s) belong to current feature must still add to perimeter
					if ((nbr8 == count) || (nbr4 == count) || (nbr2 == count) || (nbr1 == count))
						perim += OFFTable[index];
				}// for x
			}
			if (area > 0) // this feature number is in use
			{	// so store the formfactor
				feat ++;
				ff[feat] = (float)(4 * 3.1415926535897) * area / (perim * perim);
				fnum[count] = feat; // index to match value to feature ID number
				if (fref != 0) // if file was successfully opened
				{	//WriteANumber(fref, (float)feat);// these may be useful as diagnostics
					//WriteANumber(fref, area);
					//WriteANumber(fref, perim);
					WriteANumber(fref, ff[feat]);
				}
			}
		}//for count
		if (fref != 0) CloseTextFile(fref);
	} //measurement section

	// find limits for measured parameter and display results in color
	{
		float minv = (float)1e20, maxv = (float)-1e20, value;
		for (i = 1; i <= feat; i ++)
		{	
			if (ff[i] < minv) minv = ff[i];
			if (ff[i] > maxv) maxv = ff[i];
		}
		if (maxv>minv) // display feature values as hues
		{
			for (y = 0; y < height; y++)//line by line
			{
				if (TestAbortSetProgressBar((float)(height+y)/(float)(height*2)))
				{// test for cancel and advance progress bar
					result = userCanceledErr;
					goto mainexit;
				}
				ReadTempImageLine(Bin, y+1, ThisLine);//read binary
				for (x = 0; x < width; x++)
				{
					if (ThisLine[x+1] == 0)//background stays black
					{	
						OrigLine[x].red = OrigLine[x].green = OrigLine[x].blue  = 0.0;
					}	
					else
					{	
						value = 0;
						index = ThisLine[x+1]; // feature ID number
						i = fnum[index]; // convert to position in result table
						value = ff[i];// get result value
						if (value > 0) // generate a hue value and write RGB pixel
						{	value = (value - minv) / (maxv - minv); // 0..1 range
							if (value > 0.75) // red to yellow
							{	
								OrigLine[x].red = 255.0;
								OrigLine[x].green = (float)1020.0 * ((float)1.0 - value);
								OrigLine[x].blue = 0.0;
							}
							else if (value > 0.55) // yellow to green
							{	
								OrigLine[x].red = (float)1275 * (value - (float)0.55);
								OrigLine[x].green = 255.0;
								OrigLine[x].blue = 0.0;
							}
							else if (value > 0.4) // green to cyan
							{	
								OrigLine[x].red = 0.0;
								OrigLine[x].green = 255.0;
								OrigLine[x].blue = (float)1700 * ((float)0.55 - value);
							}
							else if (value > 0.2) // cyan to blue
							{	
								OrigLine[x].red = 0.0;
								OrigLine[x].green = (float)1275 * (value - (float)0.2);
								OrigLine[x].blue = 255.0;
							}
							else // blue to magenta
							{	
								OrigLine[x].red = (float)1275 * ((float)0.2 - value);
								OrigLine[x].green = 0.0;
								OrigLine[x].blue = 255.0;
							}
						// cheap and cheerful conversion from value(0..1) to hue (magenta..red)
						}
					}//else
				}// for x
				WriteResultLine(y, OrigLine);// display line of pixels with colored feature pixels
			}//for y 
		}
	}// display section

mainexit:				//clean up everything we allocated
	DisposeAPointer(OrigLine);
	DisposeAPointer(ThisLine);
	DisposeAPointer(PrevLine);
	DisposeAPointer(TempLine);
	DisposeAPointer(ff);
	DisposeAPointer(fnum);

	return result;		//pass any errors back
}//MainUserEntry

